<?php
/**
 * @package    	Domus Organizer
 * @subpackage	Tables
 * @author     	Davide Tampellini
 * @copyright 	Copyright (c)2011-2012 Davide Tampellini
 * @license 	GNU General Public License version 3, or later
 */

defined('_JEXEC') or die();

class DomusTableAttribute extends FOFTable
{
	public $default_types = array('AIR_COND', 'BUILDING_STATE', 'BUILDING_TYPE', 'EXPOSURE', 'FURNITURE',
								   'GARAGE', 'GARDEN', 'HEATING', 'HEATING_POWER', 'HEATING_SUPPLY', 'LIFT',
								   'OCCUPATION', 'PARKAREA', 'PROP_COND', 'RECEPTION', 'TOP_FLOOR');

	function __construct($table, $key, $db)
	{
		//custom table_key name
		parent::__construct($table, 'id_attributes', $db);
	}

	function onBeforeStore($updateNulls)
	{
		$result = true;

		if(!$this->att_type)
		{
			$this->setError(JText::_('COM_DOMUS_ERR_ATTRIBS_TYPE'));
			$result = false;
		}

		if(!$this->att_descr)
		{
			$this->setError(JText::_('COM_DOMUS_ERR_ATTRIBS_DESCR'));
			$result = false;
		}

		return parent::onBeforeStore($updateNulls) && $result;
	}

	function onBeforeDelete($oid)
	{
		$result = true;
		$db 	= JFactory::getDbo();

		$this->load($oid);

		$query = $db->getQuery(true)
					->select('pr_obj_id')
					->from('#__domus_prop_attributes')
					->innerJoin('#__domus_properties ON patt_id_properties = id_properties')
					->where('patt_id_attributes = '.$this->id_attributes);
		$props = $db->setQuery($query)->loadColumn();

		if($props)
		{
			$this->setError(JText::_('COM_DOMUS_ERR_ATTRIBS_IN_USE').' '.implode(', ', $props));
			$result = false;
		}

		//if type is a default one, make sure we're not deleting the last one,
		//since they are required
		if(in_array($this->att_type, $this->default_types))
		{
			$query = $db->getQuery(true)
						->select('COUNT(*)')
						->from('#__domus_attributes')
						->where('att_type = '.$db->quote($this->att_type));
			$count = $db->setQuery($query)->loadResult();

			if($count == 1)
			{
				$this->setError(JText::_('COM_DOMUS_ERR_ATTRIBS_LAST_ONE'));
				$result = false;
			}
		}

		return parent::onBeforeDelete($oid) && $result;
	}
}