<?php
/**
 * @package    	Domus Organizer
 * @subpackage	Tables
 * @author     	Davide Tampellini
 * @copyright 	Copyright (c)2011-2012 Davide Tampellini
 * @license 	GNU General Public License version 3, or later
 */

defined('_JEXEC') or die();

class DomusTableProperty extends FOFTable
{
	function __construct($table, $key, $db)
	{
		//custom table_key name
		parent::__construct($table, 'id_properties', $db);

		$this->setColumnAlias('title', 'pr_title');
		$this->setColumnAlias('slug',  'pr_alias');
		$this->setColumnAlias('locked_by', 'pr_locked_by');
		$this->setColumnAlias('locked_on', 'pr_locked_on');
	}

	function onBeforeBind($from)
	{
		//stores previous category, so I can subtract it from category entries
		if($from['id_properties'])
		{
			$db = JFactory::getDbo();

			$query = $db->getQuery(true)
						->select('pr_id_categories')
						->from('#__domus_properties')
						->where('id_properties = '.$from['id_properties']);
			$this->_prevCat = $db->setQuery($query)->loadResult();
		}

		return parent::onBeforeBind($from);
	}

	function onBeforeStore($updateNulls)
	{
		require_once JPATH_COMPONENT.'/helpers/format.php';

		$result = true;

		if(!$this->pr_obj_id)
		{
			$this->setError(JText::_('COM_DOMUS_ERR_PROP_OBJID'));
			$result = false;
		}
		else
		{
			$db = JFactory::getDbo();

			//check if property obj id is unique
			$query = $db->getQuery(true)
						->select('COUNT(*)')
						->from('#__domus_properties')
						->where('pr_obj_id = '.$db->q($this->pr_obj_id))
						->where('id_properties != '. (int) $this->id_properties);
			$count = $db->setQuery($query)->loadResult();

			if($count)
			{
				$this->setError(JText::_('COM_DOMUS_ERR_PROP_OBJID_UNIQUE'));
				$result = false;
			}
		}

		if(!$this->pr_id_categories)
		{
			$this->setError(JText::_('COM_DOMUS_ERR_PR_ID_CATEGORIES'));
			$result = false;
		}

		if(!$this->pr_id_types)
		{
			$this->setError(JText::_('COM_DOMUS_ERR_PR_ID_TYPES'));
			$result = false;
		}

		if(!$this->pr_id_districts)
		{
			$this->setError(JText::_('COM_DOMUS_ERR_PR_ID_DISTRICTS'));
			$result = false;
		}

		if(!$this->pr_id_towns)
		{
			$this->setError(JText::_('COM_DOMUS_ERR_PR_ID_TOWNS'));
			$result = false;
		}

		//at the moment always recreate the alias
		$this->pr_alias = '';

		//create alias from obj_id if there is no title
		if(!$this->pr_title) $this->pr_alias = $this->pr_obj_id;

		//convert price into database format
		$this->pr_price 	= DomusHelperFormat::priceToDatabase($this->pr_price);
		$this->pr_price_min = DomusHelperFormat::priceToDatabase($this->pr_price_min);

		//handle checkboxes (if they're not set, there is no way to uncheck them)
		$this->pr_negotiable 	 = FOFInput::getInt('pr_negotiable', 0, null);
		$this->pr_private_treaty = FOFInput::getInt('pr_private_treaty', 0, null);
		$this->pr_exclusive 	 = FOFInput::getInt('pr_exclusive', 0, null);
		$this->enabled 			 = FOFInput::getInt('enabled', 0, null);
		$this->pr_hot 			 = FOFInput::getInt('pr_hot', 0, null);

		// if the property is not avaible internally, it won't be on public side, too
		if(!$this->pr_domus_published)
		{
			$this->enabled = 0;
		}

		return parent::onBeforeStore($updateNulls) && $result;
	}

	/**
	 * Updates number of entries inside category table
	 */
	function onAfterStore()
	{
		$db = JFactory::getDbo();

		if($this->_prevCat)
		{
			//let's decrement the old category
			$query = $db->getQuery(true)
						->update('#__domus_categories')
						->set('cat_entries = cat_entries - 1')
						->where('id_categories = '.$this->_prevCat);
			$db->setQuery($query)->Query();
		}

		//let's increment the new category
		$query = $db->getQuery(true)
					->update('#__domus_categories')
					->set('cat_entries = cat_entries + 1')
					->where('id_categories = '.$this->pr_id_categories);
		$db->setQuery($query)->Query();

		//clean up oddy values (cat_entries < 0)
		$query = $db->getQuery(true)
					->update('#__domus_categories')
					->set('cat_entries = 0')
					->where('cat_entries < 0');
		$db->setQuery($query)->query();

		return parent::onAfterStore();
	}

	function onAfterDelete($oid)
	{
		$db = JFactory::getDbo();

		//decrement category count
		$query = $db->getQuery(true)
					->update('#__domus_categories')
					->set('cat_entries = cat_entries - 1')
					->where('id_categories = '.$this->pr_id_categories);
		$db->setQuery($query)->Query();

		//delete images (uses the table, so it will handle the delete logic)
		$query = $db->getQuery(true)
					->select('id_images')
					->from('#__domus_images')
					->where('img_id_properties = '.$oid);
		$id_images = $db->setQuery($query)->loadColumn();

		$image = FOFTable::getAnInstance('Image', 'DomusTable');

		foreach($id_images as $id_image)
		{
			$image->delete($id_image);
		}

		//property attributes
		$query = $db->getQuery(true)
					->delete('#__domus_prop_attributes')
					->where('patt_id_properties = '.$oid);
		$rc = $db->setQuery($query)->query();

		//property owners
		$query = $db->getQuery(true)
					->delete('#__domus_prop_owners')
					->where('pow_id_properties = '.$oid);
		$rc = $db->setQuery($query)->query();

		//property rooms
		$query = $db->getQuery(true)
					->delete('#__domus_prop_rooms')
					->where('pro_id_properties = '.$oid);
		$rc = $db->setQuery($query)->query();

		return true;
	}
}