<?php
/**
 * @package    	Domus Organizer
 * @subpackage 	Helpers
 * @author     	Davide Tampellini
 * @copyright 	Copyright (c)2011-2012 Davide Tampellini
 * @license 	GNU General Public License version 3, or later
 */

defined('_JEXEC') or die();

class DomusHelperChecks
{
	/**
	 * Runs all system checks. If anything goes wrong, return an associative array with alerts
	 *
	 * @return  array  An associative array, which keys are the part with "problems"
	 */
	public static function globalChecks()
	{
		$result = true;

		$result = self::checkAgency() 		&& $result;
		$result = self::checkAttribs() 		&& $result;
		$result = self::checkCategories() 	&& $result;
		$result = self::checkDistricts() 	&& $result;
		$result = self::checkTowns() 		&& $result;
		$result = self::checkTypes() 		&& $result;
		$result = self::checkUser() 		&& $result;

		return $result;
	}

	/**
	 * Checks if exists at least an agency
	 *
	 * @return  boolean  True/False on success/failure
	 */
	public static function checkAgency()
	{
		$params = JComponentHelper::getParams('com_domus');
		$db 	= JFactory::getDbo();

		$query = $db->getQuery(true)
					->select('COUNT(*)')
					->from('#__domus_agencies');
		$result = (bool)$db->setQuery($query)->loadResult();

		$params->set('checkAgency', $result);

		return $result;
	}

	/**
	 * Checks if "attrib" table has every needed attrib
	 *
	 * @return  boolean  True/False on success/failure
	 */
	public static function checkAttribs()
	{
		$params = JComponentHelper::getParams('com_domus');
		$db 	= JFactory::getDbo();

		//fetch required attribs from attrib table
		$req	= FOFTable::getAnInstance('Attribute', 'DomusTable')->default_types;

		$query  = $db->getQuery(true)
					 ->select('*')
					 ->from('#__domus_attributes');
		$rows = $db->setQuery($query)->loadObjectList('att_type');

		$keys = array_keys($rows);
		$diff = array_diff($req, $keys);

		if(count($diff) == 0)
		{
			$params->set('checkAttribs', true);
			return true;
		}
		else
		{
			$params->set('checkAttribs', $diff);
			return false;
		}
	}

	/**
	 * Checks if "category" table is populated
	 *
	 * @return  boolean  True/False on success/failure
	 */
	public static function checkCategories()
	{
		$params = JComponentHelper::getParams('com_domus');
		$db 	= JFactory::getDbo();

		$query = $db->getQuery(true)
					->select('COUNT(*)')
					->from('#__domus_categories');
		$result = (bool)$db->setQuery($query)->loadResult();

		$params->set('checkCategories', $result);

		return $result;
	}

	/**
	 * Checks if "district" table is populated
	 *
	 * @return  boolean  True/False on success/failure
	 */
	public static function checkDistricts()
	{
		$params = JComponentHelper::getParams('com_domus');
		$db 	= JFactory::getDbo();

		$query = $db->getQuery(true)
					->select('COUNT(*)')
					->from('#__domus_districts');
		$result = (bool)$db->setQuery($query)->loadResult();

		$params->set('checkDistricts', $result);

		return $result;
	}

	/**
	 * Checks if "town" table is populated
	 *
	 * @return  boolean  True/False on success/failure
	 */
	public static function checkTowns()
	{
		$params = JComponentHelper::getParams('com_domus');
		$db 	= JFactory::getDbo();

		$query = $db->getQuery(true)
					->select('COUNT(*)')
					->from('#__domus_towns');
		$result = (bool)$db->setQuery($query)->loadResult();

		$params->set('checkTowns', $result);

		return $result;
	}

	/**
	 * Checks if "type" table is populated
	 *
	 * @return  boolean  True/False on success/failure
	 */
	public static function checkTypes()
	{
		$params = JComponentHelper::getParams('com_domus');
		$db 	= JFactory::getDbo();

		$query = $db->getQuery(true)
					->select('COUNT(*)')
					->from('#__domus_types');
		$result = (bool)$db->setQuery($query)->loadResult();

		$params->set('checkTypes', $result);

		return $result;
	}

	/**
	 * Check if current user has every information needed
	 *
	 * @return  boolean  True/False on success/failure
	 */
	public static function checkUser()
	{
		$params = JComponentHelper::getParams('com_domus');
		$db 	= JFactory::getDbo();
		$userid = JFactory::getUser()->id;

		//is user already checked?
		$checked = $params->get('checkedUsers', new stdClass());
		if(isset($checked->$userid)) return true;

		$query = $db->getQuery(true)
					->select('COUNT(*)')
					->from('#__domus_users')
					->where('us_id_joomla = '.$userid);
		$result = (bool)$db->setQuery($query)->loadResult();

		//if the user is in Domus table, we assume that everything is ok (name, surname ecc ecc)
		if($result)
		{
			$checked->$userid = 1;
			$params->set('checkedUsers', $checked);
		}

		$params->set('checkUsers', $result);

		return $result;
	}
}