<?php
/**
 * @package    	Domus Organizer
 * @subpackage 	Helpers
 * @author     	Davide Tampellini
 * @copyright 	Copyright (c)2011-2012 Davide Tampellini
 * @license 	GNU General Public License version 3, or later
 */

defined('_JEXEC') or die();

class DomusHelperFormat
{
	static function formatAddress($row)
	{
		$params = JComponentHelper::getParams('com_domus');

		$showAddress = $params->get('completeAddress', 0);

		if($showAddress == 1 || ($showAddress == 2 && $row->pr_exclusive))
		{
			$string = $row->st_descr.' '.$row->pr_street.' '.$row->pr_streetnr.', '.$row->to_descr;
		}
		else $string = $row->to_descr;

		return $string;
	}

	static function formatDescr($descr)
	{
		if(!$descr) return '';

		$params = JComponentHelper::getParams('com_domus');
		$limit = $params->get('descrLimit', 50);

		$words = explode(' ', $descr, $limit + 1);
		array_pop($words);
		$string = implode(' ', $words).'...';

		return $string;
	}

	static function jsFormatPrice()
	{
		$params = JComponentHelper::getParams('com_domus');

		$sign 	   = $params->get('currencySimbol', '€');
		$precision = $params->get('decimalNum', 0);
		$dec  	   = $params->get('decimalSep', '.');
		$thousands = $params->get('thousandsSep', ',');

		if($params->get('currencyPos', 1))
		{
			$prefix = '';
			$suffix = ' '.$sign;
		}
		else
		{
			$prefix = $sign.' ';
			$suffix = '';
		}

		$js = 'window.addEvent("domready", function(){
				$$(".formatCurrency").addEvent("keyup", function(){
				var amount = Number.from(this.value.replace(/[^0-9-'.preg_quote($dec).']/g, ""));
				if(amount === null) return;
				this.value = amount.format({
								decimal  : "'.$dec.'",
								group    : "'.$thousands.'",
								decimals : '.$precision.'
							})
			});
		})';

		JFactory::getDocument()->addScriptDeclaration($js);
	}

	static function formatPrice($price, $privateTreaty = 0, $simbol = true)
	{
		if($privateTreaty) return JText::_('COM_DOMUS_PRIV_TREATY');

		$params = JComponentHelper::getParams('com_domus');

		$precision 	= $params->get('decimalNum', 0);
		$decimal 	= $params->get('decimalSep', '.');
		$thousands 	= $params->get('thousandsSep', ',');

		$string = number_format($price, $precision, $decimal, $thousands);

		if($simbol)
		{
			if($params->get('currencyPos', '1'))$string = $string.' '.$params->get('currencySimbol', '€');
			else								$string = $params->get('currencySimbol', '€').' '.$string;
		}

		return $string;
	}

	static function priceToDatabase($price)
	{
		$params = JComponentHelper::getParams('com_domus');

		$dec  	   = $params->get('decimalSep', '.');

		//remove any char that's not a number or decimal separator (i don't care about thousands sep)
		$regex = '#[^0-9-'.preg_quote($dec).']#';

		$normalized = preg_replace($regex, '', $price);
		$normalized = str_replace($dec, '.', $normalized);

		return (int)$normalized;
	}
}