<?php
/**
 * @package    	Domus Organizer
 * @subpackage 	Helpers
 * @author     	Davide Tampellini
 * @copyright 	Copyright (c)2011-2012 Davide Tampellini
 * @license 	GNU General Public License version 3, or later
 */

defined('_JEXEC') or die();

class DomusHelperGallery
{
	public static function storeImage($id_prop, $fullpath)
	{
		$dest_dir 	 = str_replace("\\"			, '/', $fullpath);
		$search_path = str_replace("\\"			, '/', JPATH_SITE.'/');
		$dest_dir  	 = str_replace($search_path , '' , $dest_dir);

		$imagename = pathinfo($fullpath, PATHINFO_FILENAME);
		$ext	   = pathinfo($fullpath, PATHINFO_EXTENSION);

		$dest_dir  = str_replace($imagename.'.'.$ext, '' , $dest_dir);

		$img = FOFTable::getAnInstance('Image', 'DomusTable');

		$img->id_images 		= null;
		$img->img_id_properties	= $id_prop;
		$img->img_title 		= '';
		$img->img_descr 		= '';
		$img->img_fname 		= $imagename;
		$img->img_type 			= '.'.$ext;
		$img->img_path 			= $dest_dir;
		$img->ordering 			= 100;			#takes the last position
		$img->img_published		= 1;

		$img->store();
		$img->reorder('img_id_properties = '.$id_prop);
	}

	/**
	 * Ridimensiona una fotografia.
	 * @param int $is_thmb Deve creare un'anteprima o una foto normale? (1/0)
	 * @param string $src_file File sorgente
	 * @param string $dest_file File destinazione
	 * @param int $width Larghezza desiderata
	 * @param int $height Altezza desiderata
	 * @param bool $prop Il ridimensionamento deve essere proporzionale?
	 * @param int $quality Qualita' da applicare
	 */
	public static function resizeIMG($is_thmb, $src_file, $dest_file, $width, $height, $prop, $quality)
	{
		if(!function_exists("gd_info"))
		{
			echo 'You do not have the GD Library installed.  This class requires the GD library to function properly.' . "\n";
			return false;
		}

		$params = JComponentHelper::getParams('com_domus');

		$imagetype  = array( 1 => 'GIF', 2 => 'JPG', 3 => 'PNG' );
		$imginfo 	= getimagesize($src_file);
		$imginfo[2] = $imagetype[$imginfo[2]];

		$srcWidth  = $imginfo[0];
		$srcHeight = $imginfo[1];

		if ($prop)
		{
			$wantratio = $width / $height;
			$haveratio = $srcWidth / $srcHeight;

			if ($wantratio < $haveratio)
			{
				$destWidth  = $width;
				$destHeight = $width / $haveratio;
			}
			else
			{
				$destWidth  = $height * $haveratio;
				$destHeight = $height;
			}
		}
		else
		{
			$destWidth  = (int)($width);
			$destHeight = (int)($height);
		}

		if     ($imginfo[2] == 'JPG') $src_img = imagecreatefromjpeg($src_file);
		else if($imginfo[2] == 'GIF') $src_img = imagecreatefromgif($src_file);
		else if($imginfo[2] == 'PNG') $src_img = imagecreatefrompng($src_file);

		if(function_exists("imagecreatetruecolor"))	$dst_img = imagecreatetruecolor($destWidth, $destHeight);
		else										$dst_img = imagecreate($destWidth, $destHeight);

		if(function_exists("imagecopyresampled"))	imagecopyresampled($dst_img, $src_img, 0, 0, 0, 0, (int)$destWidth, (int)$destHeight, $srcWidth, $srcHeight);
		else										imagecopyresized($dst_img, $src_img, 0, 0, 0, 0,(int) $destWidth, (int)$destHeight, $srcWidth, $srcHeight);

		if(!$is_thmb && $params->get('watermark', 0))
		{
			$wmstr 	 = $params->get('watermarkText');

			if(!$wmstr)
			{
				$year    = JHTML::_('date', 'now', '%Y' );
				$wmstr   = '(c)'.$year.' '.JURI::root().' - powered by Domus Organizer';
			}

			$ftcolor = imagecolorallocate($dst_img,239,239,239);
			imagestring ($dst_img, 2,10, $destHeight - 20, $wmstr, $ftcolor);
		}

		imagejpeg($dst_img, $dest_file, $quality);
		imagedestroy($src_img);
		imagedestroy($dst_img);
		// Set mode of uploaded picture
		chmod($dest_file, octdec('644'));
		// We check that the image is valid
		$imginfo = getimagesize($dest_file);

		if ($imginfo == null)	return false;
		else					return true;

	}

	public static function filename_safe($filename)
	{
		$filename = strtolower($filename);
		$filename = str_replace(" ", "_", $filename);

		$result = '';
		for ($i=0; $i<strlen($filename); $i++)
		{
			if (preg_match('([0-9]|[a-z]|_)', $filename[$i])) $result = $result . $filename[$i];
		}

		return $result;
	}
}