<?php
/**
 * @package    	Domus Organizer
 * @subpackage 	Helpers
 * @author     	Davide Tampellini
 * @copyright 	Copyright (c)2011-2012 Davide Tampellini
 * @license 	GNU General Public License version 3, or later
 */

defined('_JEXEC') or die();

class DomusHelperHtml
{
	protected static $attributes;

	// Seems that *fade* transitions create some problems
	/*static $transitions = array('slideDown', 'slideUp duration:500', 'slideLeft', 'slideRight', 'crossFade',
								'pushUp duration:500', 'pushDown', 'pushLeft duration:1500', 'pushRight', 'fadeThroughBackground',
								'blindUp', 'blindDown', 'blindLeft', 'blindRight', 'blindUpFade', 'blindDownFade', 'blindLeftFade',
								'blindRightFade', 'fade');*/
	static $transitions = array('slideDown', 'slideUp duration:500', 'slideLeft', 'slideRight',
								'pushUp duration:500', 'pushDown', 'pushLeft duration:1500', 'pushRight',
								'blindUp', 'blindDown', 'blindLeft', 'blindRight');


	public static function startPane($overText = false)
	{
		#jimport('joomla.html.pane');
		$params['onActive'] = "
		function(title, description){".
		"title.addClass('open').removeClass('closed');
		description.setStyle('display', 'inline');";

		if($overText) $params['onActive'] .= " OverText.update();";

		$params['onActive'] .= "}";
		return JHTML::_('tabs.start', 'tabs', $params);
	}

	/**
	 * Translates a fixed fields that aren't inside the db (ie sex, floor etc. etc.)
	 *
	 * @param string $field
	 * @param string $value
	 *
	 * @return string Decoded value
	 */
	public static function decoFix($field, $value)
	{
		switch($field)
		{
			case 'cust_type'	:
				if    ($value == 'C') $string = JText::_('COM_DOMUS_CUSTOMER');
				elseif($value == 'O') $string = JText::_('COM_DOMUS_OWNER');
				elseif($value == 'B') $string = JText::_('COM_DOMUS_BOTH');
				break;
			case 'floor'		:
				if    ($value == -2) $string = JText::_('COM_DOMUS_MEZZANINE');
				elseif($value == -1) $string = JText::_('COM_DOMUS_BASEMENT');
				elseif($value ==  0) $string = JText::_('COM_DOMUS_GROUND_FLOOR');
				else				 $string = $value.'&deg;';
				break;
			case 'domus_published' :
			   	if    ($value ==  -1) $string = JText::_('COM_DOMUS_TRASHED');
			   	elseif($value ==   0) $string = JText::_('COM_DOMUS_UNPUBLISHED');
			   	elseif($value ==   1) $string = JText::_('COM_DOMUS_PUBLISHED');
			   	elseif($value ==   2) $string = JText::_('COM_DOMUS_SOLD');
			   	elseif($value ==   3) $string = JText::_('COM_DOMUS_RENTED');
			   	elseif($value ==   4) $string = JText::_('COM_DOMUS_POSITION_ENDED');
				break;
			case 'sex'			:
				if    ($value == 'M') $string = JText::_('COM_DOMUS_MALE');
				elseif($value == 'F') $string = JText::_('COM_DOMUS_FEMALE');
				break;
			default:
				$string = $value;
				break;
		}

		return $string;
	}

	public static function tickImg($value, $ok = 'ok_16.png', $ko = 'ko_16.png', $titleok = '', $titleko = '')
	{
		if($value)
		{
			$file  = $ok;
			$title = $titleok;
		}
		else
		{
			$file  = $ko;
			$title = $titleko;
		}

		return '<img title="'.$title.'" src="'.FOFTemplateUtils::parsePath('com_domus/images/'.$file).'" />';
	}

	public static function published($value)
	{
		#@TODO search for publish icons
		switch($value)
		{
			case  0:
			default:	$img = 'ko_16.png';	break;
			case  1:	$img = "ok_16.png";	break;
		}

		$title = self::decoFix('domus_published', $value);
		$html .= '<img src="'.FOFTemplateUtils::parsePath('com_domus/images/'.$img) .'" border="0" title="'. $title .'" />';

		return $html;
	}

	public static function echoif($value, $koecho = '', $okecho = '')
	{
		if    (!empty($value) && $okecho)	return $okecho;
		elseif(!empty($value))				return $value;
		else								return $koecho;
	}

	/**
	 * Creates a checkbox field for a property attribute field
	 *
	 * @param  string 	$type			Attribute name
	 * @param  string	$name			Checkbox name
	 * @param  int 		$checked		Is checkbox checked?
	 * @param  string 	$tag_attribs	Tag attributes
	 * @param  string 	$text			Label text
	 * @param  bool		$unique_id		Should i generate a unique id?
	 */
	public static function createAttrCbx($type, $name, $checked = null, $tag_attribs = '', $text = '', $unique_id = false)
	{
		if(!self::$attributes) self::$attributes = self::getAttributes();

		$value = self::$attributes[strtoupper($type)]->value;

		$html = self::cbx($name, $value, $checked, $tag_attribs, $text, $unique_id);

		return $html;
	}

	/**
	 * Creates a standard checkbox field
	 *
	 * @param string 	$name		 Checkbox name
	 * @param mixed 	$value		 Checkbox value
	 * @param mixed 	$checked 	 Is checkbox checked?
	 * @param string 	$tag_attribs Tag attributes
	 * @param string 	$text		 Label text
	 * @param bool		$generate_id Should I generate a unique id?
	 *
	 * @return string	Checkbox html code
	 */
	public static function cbx( $name, $value, $checked = null, $tag_attribs = '', $text = '', $generate_id = false)
	{
		if(!$generate_id)	$id = $name;
		else				$id = uniqid();

		$t 		 = ($text)		? '&nbsp;<label class="pointer" for="'.$id.'">'.$text.'</label>' : '';
		$checked = ($checked)	? ' checked="checked"' : '';

		$html 	 = '<input type="checkbox" id="'.$id.'" name="'.$name.'" '.$tag_attribs.' value="'.$value.'" '.$checked.' />'.$t;
		return $html;
	}

	/**
	 * Creates a sort link, compatible with Joomla! SEF
	 *
	 * @param  string  $text       Link text
	 * @param  string  $field      Order field
	 * @param  string  $direction  Current direction
	 * @param  string  $selected   Selected field
	 *
	 * @return string  Sort html
	 */
	public static function gridSort($text, $field, $direction, $selected)
	{
		$html = JHTML::_('grid.sort', JText::_($text).'&nbsp;', $field, $direction, $selected);
		$html = str_replace('href="#"', 'style="cursor:pointer;text-decoration:underline"', $html);
		return $html;
	}

	/**
	 * Creates a clickable sort link, outside a form (ie using GET instead of submitting a form)
	 *
	 * @param  string $order
	 * @param  string $dir
	 * @param  string $field
	 *
	 * @return string Sort html, ready to be parsed with sprintf. First token is the link, the second
	 *                one is the text link.
	 */
	public static function noformSort($order, $dir, $field)
	{
		// The order field is the one I'm looking for
		if($field == $order)
		{
			$new_dir   = strtoupper($dir) == 'DESC' ? 'ASC' : 'DESC';
			$source    = FOFTemplateUtils::parsePath('system/images/sort_'.strtolower($dir).'.png');
			$image_dir = '<img src="'.$source.'" />';
		}
		else $new_dir = 'ASC';

		$filter = '&filter_order='.$field.'&filter_order_Dir='.$new_dir;

		return '<a href="%s'.$filter.'">%s&nbsp;'.$image_dir.'</a>';
	}

	public static function checkedOut($row, $i, $identifier, $type, $createCbx = true)
	{
		$userid = JFactory::getUser()->get('id');
		$table  = FOFTable::getAnInstance($type, 'DomusTable');
		$locked = $table->getColumnAlias('locked_by');

		$result = $table->isCheckedOut($userid, $row->$locked);

		//I just want to check if the record is checked out
		if(!$createCbx)	return $result;

		if ($result) $checked = self::_checkedOut($row, 1, $type);
		else		 $checked = JHtml::_('grid.id', $i, $row->$identifier);

		return $checked;
	}

	public static function propLoadFirstPic($id_prop, $width, $attrib = '', $admin = 1)
	{
		$db 	= JFactory::getDbo();
		$params = JComponentHelper::getParams('com_domus');

		$file = $params->get('defaultNoImg', 'media/com_domus/images/nopic.png');

		$query = FOFQueryAbstract::getNew($db)
				 ->select('*')
				 ->from('#__domus_images')
				 ->where('img_id_properties = '.$id_prop)
				 ->where("(img_type = '.jpg' OR img_type = '.gif' OR img_type = '.png')")
				 ->order('ordering ASC');

		if(!$admin) $query->where('img_published = 1');

		$image = $db->setQuery($query, 0 ,1)->loadObject();

		if($image)
		{
			$picpath = $image->img_path;

			$p1 = 'thumbs/t_'.$image->img_fname.'.jpg';
			$p3 = $image->img_fname.$image->img_type;

			if(substr($picpath, 0, 1) != '/') $picpath = '/'.$picpath;
			$thumb = JPATH_SITE.$picpath.$p1;
			$img   = JPATH_SITE.$picpath.$p3;

			if    (JFile::exists($thumb))	$file = $picpath.$p1;
			elseif(JFile::exists($img))		$file = $picpath.$p3;

			$file = str_replace('\\', '/', $file);
			if(substr($file, 0, 1) == '/') $file = substr($file, 1);

			$html =	'<img src="'.JURI::root().$file.'" '.$attrib.' width="'.$width.'" title="'.$image->pr_title.'" />';
		}
		else
		{
			$picpath = JURI::root().$file;
			$html =	'<img src="'.$picpath.'" width="'.$width.'" />';
		}

		return $html;
	}

	static function  createSlideShow($id_prop, $thumb = true)
	{
		jimport('joomla.environment.browser');
		require_once JPATH_COMPONENT.'/helpers/property.php';
		$doc	= JFactory::getDocument();

		$images = DomusHelperProperty::getImages($id_prop, true);

		if(!$images) return '';
		$browser = new JBrowser();

		//workaround flag for ie7
		if(strtoupper($browser->getBrowser()) == 'MSIE' && $browser->getMajor()) $ie7 = true;

		FOFTemplateUtils::addJS('com_domus/library/slideshow/Loop.js');
		FOFTemplateUtils::addJS('com_domus/library/slideshow/SlideShow.js');
		FOFTemplateUtils::addJS('com_domus/library/milkbox/milkbox-yc.js');
		FOFTemplateUtils::addCSS('com_domus/library/milkbox/milkbox.css');

		FOFTemplateUtils::addJS('com_domus/js/slideshow.js');

		$trans = self::$transitions;
		shuffle($trans);
		$i = 0;

		if($thumb)
		{
			$html .= '<div id="thumbs" class="sx w40_">';

			foreach($images as $img)
			{
				$i == 0 ? $class = 'current' : $class = '';
				$path = $img->img_path.'thumbs/t_'.$img->img_fname.'.jpg';

				$html .= '<img class="thumb '.$class.'" src="'.$path.'" />';
				$i++;
			}

			$html .= '</div>';
		}

		$html .= '<div id="slideshowHolder">'."\n";
		$html .= 	'<div id="slideshow" class="slideshow" data-slideshow="delay:4000 duration:1000 transition:fade">'."\n";
		$i = 0;
		foreach($images as $img)
		{
			if($i == count($trans)) $i = 0;

			$path  = $img->img_path.$img->img_fname.$img->img_type;
			$title = '<h4>'.$img->img_title.'</h4><div>'.$img->img_descr.'</div>';


			$html .= 	'<a href="'.$path.'" data-milkbox="gallery" data-slideshow="transition:'.$trans[$i].'"';

			// Sadly ie7 + milkbox + title won't work :(
			if(!$ie7) $html .= 'title="'.$title.'"';

			$html .= 	'>';
			$html .= 		'<img src="'.$path.'" />'."\n";
			$html .=	'</a>';
			$i++;
		}
		$html .= 	'</div>'."\n";
		$html .= '</div>'."\n";

		return $html;
	}

	/**
	 * Fetches the attributes from database
	 *
	 * @return array	Associative array of attributes
	 */
	protected static function getAttributes()
	{
		$db = JFactory::getDbo();

		$query = $db->getQuery(true)
					->select('id_attributes as value, att_descr as text, att_type')
					->from('#__domus_attributes');

		$rows = $db->setQuery($query)->loadObjectList('att_type');
		return $rows;
	}

	protected static function _checkedOut(&$row, $overlib = 1, $type = '')
	{
		if ($overlib)
		{
			JHTML::_('behavior.tooltip');
			$table = FOFTable::getAnInstance($type, 'DomusTable');
			$by	   = $table->getColumnAlias('locked_by');
			$lock  = $table->getColumnAlias('locked_on');

			$date  = JHtml::_('date', $row->$lock, JText::_('DATE_FORMAT_LC1'));
			$time  = JHtml::_('date', $row->$lock, 'H:i');

			$user  = FOFModel::getAnInstance('User', 'DomusModel')
						->us_id_joomla($row->$by)
						->getList(true);

			$hover  =  '<span class="editlinktip hasTip" title="'.JText::_('JLIB_HTML_CHECKED_OUT').'::';
			$hover .= htmlentities($user[0]->us_name.' '.$user[0]->us_surname).'<br />'.$date.'<br />'.$time.'">';
		}

		$checked = $hover.JHtml::_('image', 'admin/checked_out.png', null, null, true).'</span>';

		return $checked;
	}
}