<?php
/**
 * @package    	Domus Organizer
 * @subpackage 	Helpers
 * @author     	Davide Tampellini
 * @copyright 	Copyright (c)2011-2012 Davide Tampellini
 * @license 	GNU General Public License version 3, or later
 */

defined('_JEXEC') or die();

class FBModelHelper
{
	/**
	 * Method to save more than a record without using the model. Usually used when
	 * saving a master/detail page.
	 * In order to save rows with checkboxes, we need to create progressive names to these fields
	 * (i.e. 'my_field1', 'my_field2' etc  etc). We can't use an array name (i.e. 'my_field[]') since
	 * unchecked fields are not submitted, and this would mess up the stack.
	 *
	 * @param  string  $jtable       JTable name
	 * @param  array   $fixedFields  Associative array (i.e. 'name' => 'value') containing fields
	 * 								 with a fixed value
	 * @param  array   $cbxFields    Array containing checkbox field names
	 * @param  string  $tbprefix     Table prefix, default 'JTable'
	 *
	 * @return  boolean  True on success, false otherwise (setting the error, too)
	 */
	static function saveMultipleRows($jtable, $fixedFields = array(), $cbxFields = array(), $tbprefix = 'FOFTable')
	{
		$data  = JRequest::get('post');
		$table = FOFTable::getAnInstance($jtable, $tbprefix);

		$pk = $table->getKeyName();

		$fields = $table->getTableFields();

		$merged = array_merge($fields, $data);

		for($i = 0; $i < count($data[$pk]); $i++)
		{
			$bind = array();
			$bind[$pk] = (int)$data[$pk][$i];

			//collego i vari campi della tabella ai valori passati
			foreach($fields as $field => $value)
			{
				//primary key, i already set that
				if($field == $table->getKeyName()) continue;

				//field with fixed value?
				if(array_key_exists($field, $fixedFields))
				{
					$bind[$field] = $fixedFields[$field];
				}
				//checkbox field?
				elseif(in_array($field, $cbxFields))
				{
				//if so i try to guess the name
					$bind[$field] = (int)$data[$field.$i][0];
				}
				else
				{
					$bind[$field] = $data[$field][$i];
				}
			}

			//at the moment I don't handle errors, simply skip the record
			if(!$table->bind($bind)) 	continue;
			if(!$table->check())		continue;
			if(!$table->store())		continue;
		}

		return true;
	}
}