<?php
/**
 * @package    	Domus Organizer
 * @subpackage 	Helpers
 * @author     	Davide Tampellini
 * @copyright 	Copyright (c)2011-2012 Davide Tampellini
 * @license 	GNU General Public License version 3, or later
 */

defined('_JEXEC') or die();

class DomusHelperSelect
{
	private static $attributes;
	static $options = array();

	static function createTownList($field, $district, $value = '', $attrib = '')
	{
		$sql = array('table' => 'domus_towns',
					 'id'	 => 'id_towns',
					 'descr' => 'to_descr',
					 'where' => 'to_id_districts = '.$district);
		$opt  = self::buildSelectOptions($sql);
		$html = self::createSelectList($opt, $field, $value, $attrib);

		return $html;
	}

	static function createZoneList($field, $town, $value = '', $attrib = '')
	{
		$sql = array('table' => 'domus_zones',
					 'id'	 => 'id_zones',
					 'descr' => 'zo_descr',
					 'where' => 'zo_id_towns = '.$town);
		$opt  = self::buildSelectOptions($sql);
		$html = self::createSelectList($opt, $field, $value, $attrib);

		return $html;
	}

	static function createSzoneList($field, $zone, $value = '', $attrib = '')
	{
		$sql = array('table' => 'domus_szones',
					 'id'	 => 'id_szones',
					 'descr' => 'sz_descr',
					 'where' => 'sz_id_zones = '.$zone);
		$opt  = self::buildSelectOptions($sql);
		$html = self::createSelectList($opt, $field, $value, $attrib);

		return $html;
	}

	/**
	 * Create a select list
	 * @param string $type 		List to create
	 * @param string $field 	Field name
	 * @param string $value 	Selected value
	 * @param string $attrib 	Select list attrib
	 * @param string $select 	Text for the first option without value.
	 * @param string $no_value 	Value for the first option.
	 *
	 * @return HTML select list
	 */
	static function createList($type, $field, $value = '', $attrib = '', $select = '', $no_value = '')
	{
		if(!$select) $select = JText::_('COM_DOMUS_SELECT_LIST');

		if(self::$options[strtoupper($type)]) $opt = self::$options[strtoupper($type)];
		else
		{
			switch(strtolower($type))
			{
				case 'air_cond'	 	 :
				case 'building_state':
				case 'building_type' :
				case 'exposure'	 	 :
				case 'furniture' 	 :
				case 'garage'		 :
				case 'garden'		 :
				case 'heating'	 	 :
				case 'heating_power' :
				case 'heating_supply':
				case 'occupation'	 :
				case 'parkarea'		 :
				case 'prop_cond'	 :

					if(!self::$attributes) self::$attributes = self::getAttributes();

					$opt = self::$attributes[strtoupper($type)];
					array_unshift($opt, array('value' => '', 'text' => $select));

					break;

				case 'categories': $opt   = self::catSelectList();	break;
				case 'cat_prop'	 :
									$model = FOFModel::getTmpInstance('Categories', 'DomusModel');
									$cats  = $model->getCatsFromProps();
									$opt   = self::catSelectList(false, true, $cats);
									break;
				case 'cust_group': 	$sql = array('table' => 'domus_customer_groups', 'id' => 'id_customer_groups', 'descr' => 'cg_descr'); break;
				case 'cust_type' :  $opt[] = array('value' => '' , 'text' => $select);
									$opt[] = array('value' => 'C', 'text' => JText::_('COM_DOMUS_CUSTOMER'));
									$opt[] = array('value' => 'O', 'text' => JText::_('COM_DOMUS_OWNER'));
									$opt[] = array('value' => 'B', 'text' => JText::_('COM_DOMUS_BOTH'));
									break;
				case 'distr_public'	: $sql = array( 'table' => 'domus_towns, #__domus_properties, #__domus_districts',
													'id'	=> 'id_districts',
													'descr'	=> 'di_descr',
													'where'	=> 'pr_id_towns = id_towns AND to_id_districts = id_districts');
									  break;
				case 'district'	 : $sql = array('table' => 'domus_districts',
												'id'	=> 'id_districts',
												'descr' => 'di_descr');
									break;

				case 'floor'	 :	$opt[] = JHTML::_('select.option', '' , $select);
									$opt[] = JHTML::_('select.option', '-2' , JText::_('COM_DOMUS_MEZZANINE'));
									$opt[] = JHTML::_('select.option', '-1' , JText::_('COM_DOMUS_BASEMENT'));
									$opt[] = JHTML::_('select.option', '0'  , JText::_('COM_DOMUS_GROUND_FLOOR'));

									for($i = 1; $i <= 25; $i++){
										$opt[] = JHTML::_('select.option', $i , $i.'&deg; '.strtolower(JText::_('COM_DOMUS_FLOOR')));
									}
									break;

				case 'prop_attr' : 	$opt[] = JHTML::_('select.option',   ''				, $select);
									$opt[] = JHTML::_('select.option',   'AIR_COND'		, JText::_('COM_DOMUS_AIR_COND'));
									$opt[] = JHTML::_('select.option',   'BUILDING_STATE', JText::_('COM_DOMUS_BUILDING_STATE'));
									$opt[] = JHTML::_('select.option',   'BUILDING_TYPE', JText::_('COM_DOMUS_BUILDING_TYPE'));
									$opt[] = JHTML::_('select.option',   'EXPOSURE'		, JText::_('COM_DOMUS_EXPOSURE'));
									$opt[] = JHTML::_('select.option',   'FURNITURE'	, JText::_('COM_DOMUS_FURNITURE'));
									$opt[] = JHTML::_('select.option',   'GARAGE'		, JText::_('COM_DOMUS_GARAGE'));
									$opt[] = JHTML::_('select.option',   'GARDEN'		, JText::_('COM_DOMUS_GARDEN'));
									$opt[] = JHTML::_('select.option',   'HEATING'		, JText::_('COM_DOMUS_HEATING'));
									$opt[] = JHTML::_('select.option',   'HEATING_POWER', JText::_('COM_DOMUS_HEATING_POWER'));
									$opt[] = JHTML::_('select.option',   'HEATING_SUPPLY', JText::_('COM_DOMUS_HEATING_SUPPLY'));
									$opt[] = JHTML::_('select.option',   'LIFT'			, JText::_('COM_DOMUS_LIFT'));
									$opt[] = JHTML::_('select.option',   'OCCUPATION'	, JText::_('COM_DOMUS_OCCUPATION'));
									$opt[] = JHTML::_('select.option',   'PARKAREA'		, JText::_('COM_DOMUS_PARKAREA'));
									$opt[] = JHTML::_('select.option',   'PROP_COND'	, JText::_('COM_DOMUS_PROP_COND'));
									$opt[] = JHTML::_('select.option',   'RECEPTION'	, JText::_('COM_DOMUS_RECEPTION'));
									$opt[] = JHTML::_('select.option',   'TOP_FLOOR'	, JText::_('COM_DOMUS_TOP_FLOOR'));

									JArrayHelper::sortObjects($opt, 'text');

									break;

				case 'publish'   : 	$opt[] = array('value' =>   '', 'text' => $select);
									$opt[] = array('value' => '-1', 'text' => JText::_('COM_DOMUS_TRASHED'));
									$opt[] = array('value' =>  '0', 'text' => JText::_('COM_DOMUS_UNPUBLISHED'));
									$opt[] = array('value' =>  '1', 'text' => JText::_('COM_DOMUS_PUBLISHED'));
									$opt[] = array('value' =>  '2', 'text' => JText::_('COM_DOMUS_SOLD'));
									$opt[] = array('value' =>  '3', 'text' => JText::_('COM_DOMUS_RENTED'));
									$opt[] = array('value' =>  '4', 'text' => JText::_('COM_DOMUS_POSITION_ENDED'));
									break;

				case 'rooms'	 : $sql   = array('table' => 'domus_rooms',
												'id'	  => 'id_rooms',
												'descr'   => 'ro_descr');
									break;

				case 'sex'	 	 : 	$opt[] = array('value' => '' , 'text' => $select);
									$opt[] = array('value' => 'M', 'text' => JText::_('COM_DOMUS_MALE'));
									$opt[] = array('value' => 'F', 'text' => JText::_('COM_DOMUS_FEMALE'));
									break;

				case 'source'	   : $sql  = array('table' => 'domus_sources', 'id' => 'id_sources', 'descr' => 'so_descr'); break;
				case 'spaceMeasure':$opt[] = array('value' => 'm&sup2;', 'text' => 'm&sup2; standard');
									$opt[] = array('value' => 'ft&sup2;', 'text' => 'ft&sup2; imperial');
									break;

				case 'street_type': $sql   = array(	'table' => 'domus_street_type',
													'id'	=> 'id_street_type',
													'descr' => 'st_descr');
													break;
				case 'szone'	 :
				case 'town'		 :
				case 'zone'		 :
									$opt[] = array('value' => '' , 'text' => $select); break;
				case 'type'	 	 :  $sql   = array('table' => 'domus_types', 'id' => 'id_types', 'descr' => 'ty_name');	break;

				#case 'town_all'
				case 'town_public' 	: $sql = array( 'table' => 'domus_towns, #__domus_properties',
													'id'	=> 'id_towns',
													'descr'	=> 'to_descr',
													'where'	=> 'pr_id_towns = id_towns');
									  break;

				case 'domus_users'	 : $sql	  = array('table' 	 => 'domus_users',
														'id' 	 => 'id_users',
														'descr'  => "CONCAT(us_surname, ' ', us_name)");
														break;

				default			 : $opt[] = array('value' => '' , 'text' => $select); break;
			}

			if($sql) $opt  = self::buildSelectOptions($sql, $select, $no_value);
			self::$options[strtoupper($type)] = $opt;
		}
		$list = self::createSelectList($opt, $field, $value, $attrib);

		return $list;
	}

	static function getAttributes()
	{
		$db = JFactory::getDbo();

		$query = $db->getQuery(true)
					->select('id_attributes as value, att_descr as text, att_type')
					->from('#__domus_attributes');

		$rows = $db->setQuery($query)->loadObjectList();

		foreach($rows as $row)
		{
			$return[$row->att_type][] = $row;
		}

		return $return;
	}

	/**
	 * Builds JHTML options starting from a sql select
	 *
	 * @todo  Convert the array in order to accept a query object
	 *
	 * @param  array   $sql        Array with select parts
	 * @param  string  $no_option  No option text [optional]
	 * @param  string  $no_value   No option value [optional]
	 *
	 * @return  array  Select options, ready to use with JHTML::_('select.genericlist')
	 */
	static function buildSelectOptions($sql, $no_option = '', $no_value = '')
	{
		$db = JFactory::getDBO();

		if(!$no_option) $no_option = JText::_('COM_DOMUS_SELECT_LIST');

		$id    = isset($sql['id']	) ? $sql['id'] 	  : 'id';
		$descr = isset($sql['descr']) ? $sql['descr'] : 'descr';

		$query = "SELECT DISTINCT $id as id, $descr as descr";
		if(isset($sql['alt'])) $query .= ", ".$sql['alt']." as alt ";
		$query .= " FROM #__{$sql['table']} WHERE 1 = 1";

		if(isset($sql['where']))
			$query .= " AND {$sql['where']}";

		$query .= isset($sql['order']) ? " ORDER BY {$sql['order']}" : " ORDER BY 2";

		$db->setQuery($query);
		$risultati = $db->loadObjectList();

		$options[] = JHTML::_('select.option', $no_value, JText::_("$no_option"));
		if(!count($risultati)) return $options;

		foreach($risultati as $row){
			if(isset($sql['show_id'])){
				if(isset($sql['alt']))	$descr = $row->alt.'  '.$row->descr;
				else					$descr = $row->id.'  '.$row->descr;
			}
			else						$descr = $row->descr;

			$options[] = JHTML::_('select.option', $row->id, $descr);
		}

		return $options;
	}

	/**
	 * Crea una select list selezionando correttamente il valore iniziale anche quando è negativo.
	 * JHTML non riesce a fare questo perchè fa un controllo $valore ? '' : '' e quindi i negativi (o 0)
	 * non passano il test
	 * @param array $options Array creato con {@see JHTMLSelect::Options()}
	 * @param string $campo Nome del campo
	 * @param string/int $valore Valore selezionato di default
	 * @param string $attributi Attributi per l'elemento select
	 * @param string $tipo Tipo di lista. Di default select.genericlist
	 * @return string Codice HTML per la select list
	 */
	static function createSelectList($options, $field, $value, $attribs = null, $tipo = 'select.genericlist')
	{
		$opt['id'] 			= $field;
		$opt['list.attr'] 	= $attribs;
		$opt['option.key'] 	= 'value';
		$opt['option.text'] = 'text';
		$opt['option.key.toHtml'] = '';
		$opt['option.text.toHtml'] = '';
		$opt['list.select'] = $value;

		$list = JHTML::_($tipo, $options, $field, $opt);

		//il valore da selezionare è negativo. JHTML non riesce a selezionare i valori negativi
		if(!$value)
		{
			//annullo eventuali valori già selezionati
			$list = str_replace('selected="selected"', '', $list);
			$list = str_replace('value="'.$value.'"', 'value="'.$value.'" selected="selected"', $list);
		}

		return $list;
	}

	/**
	 * Method to get a kind-of-tree of categories.
	 * NOTE: It's not a fully recursive tree!
	 *
	 * @param boolean $object    Switch to get a select list or a tree object
	 * @param boolean $published Switch to get published categories only. WHATCH OUT! WORKS ON THE CONTRARY!!
	 * @param array   $cats      Subset of categories from which to derive the tree
	 *
	 * @return mixed
	 */
	static function catSelectList($object = false, $published = true, $cats = array())
	{
		$whiteList = array();

		$db = JFactory::getDBO();

		$query = $db->getQuery(true);
		$query->select('*')
			  ->select('CASE WHEN CHAR_LENGTH(cat_alias) THEN CONCAT_WS(":", id_categories, cat_alias) ELSE id_categories END as catslug')
			  ->from('#__domus_categories')
			  ->order('cat_parent ASC, ordering ASC');

		if(!$published) $whiteList = self::getPublishedCats();
		if($cats)		$whiteList = array_merge($whiteList, $cats);
		if(count($whiteList)) $query->where('id_categories IN('.implode(',', $whiteList).')');

		$rows = $db->setQuery($query)->loadObjectList('id_categories');

		foreach($rows as $row)
		{
			$parent[$row->cat_parent][] = $row->id_categories;
		}

		if(!$object)
		{
			$options[0]	= JHTML::_('select.option', '', JText::_('COM_DOMUS_SELECT_LIST'));
			$options = array_merge($options, self::createCatOptions(0, '', $parent, $rows));
			$return = $options;
		}
		else
		{
			$return = self::createCatTree(0, $parent, $rows);
		}

		return $return;
	}

	/**
	 * Method to return the list of published categories. To be here both the category
	 * AND the parent must be published.
	 *
	 * @return array Published categories id
	 */
	static function getPublishedCats()
	{
		static $publishedCats;

		$db = JFactory::getDbo();

		//let's cache the results
		if($publishedCats) return $publishedCats;

		$query = $db->getQuery(true);
		$query->select('a.id_categories')
				->from('#__jd_categories a')
				->leftJoin('#__jd_categories b ON (a.cat_parent = b.id_categories)')
				->where('a.cat_published = 1')
				->where('(b.cat_published = 1 OR b.cat_published IS NULL)');
		$publishedCats = $db->setQuery($query)->loadResultArray();

		return $publishedCats;
	}

	static function createCatOptions($parent, $prefix, $parents, $rows)
	{
		$cats = $parents[$parent];

		if(!$cats) return false;

		foreach($cats as $cat)
		{
			$opt[] 	  = JHTML::_('select.option', $rows[$cat]->id_categories, $prefix."->".$rows[$cat]->cat_name);
			$children = self::createCatOptions($rows[$cat]->id_categories, $prefix.'&nbsp;&nbsp;&nbsp;', $parents, $rows);

			if($children) $opt = array_merge($opt, $children);
		}

		return $opt;
	}

	static function createCatTree($parent, $parents, $rows)
	{
		$cats = $parents[$parent];

		if(!$cats) return array();

		foreach($cats as $cat)
		{
			$items[$cat] = $rows[$cat];
			$items[$cat]->children = self::createCatTree($rows[$cat]->id_categories, $parents, $rows);
		}

		return $items;
	}
}