<?php
/**
 * @package    	Domus Organizer
 * @subpackage 	Models
 * @author     	Davide Tampellini
 * @copyright 	Copyright (c)2011-2012 Davide Tampellini
 * @license 	GNU General Public License version 3, or later
 */

defined('_JEXEC') or die();

class DomusModelCategories extends FOFModel
{
	public function buildQuery($overrideLimits = false)
	{
		$db = JFactory::getDbo();

		$query = $db->getQuery(true)
					->select('a.*, b.cat_name as parent_name')
				  	->select('CASE WHEN CHAR_LENGTH(a.cat_alias) THEN CONCAT_WS(":", a.id_categories, a.cat_alias) ELSE a.id_categories END as catslug')
				  	->from('#__domus_categories a')
				  	->leftJoin('#__domus_categories b ON a.cat_parent = b.id_categories');

		return $query;
	}

	public function onAfterSave($table)
	{
		$this->otable = $table;
		$this->reorder();
	}

	public function getReorderWhere()
	{
		return 'cat_parent = '.$this->otable->cat_parent;
	}

	/**
	 * Return categories (and their parents) currently used by properties.
	 *
	 * @return array Used categories
	 */
	public function getCatsFromProps()
	{
		$db = JFactory::getDbo();

		$query = $db->getQuery(true)
					->select('DISTINCT id_categories')
					->from('#__domus_categories cats')
					->innerJoin('#__domus_properties ON pr_id_categories = id_categories')
					->order('cat_parent ASC, cats.ordering ASC');

		$rows = $db->setQuery($query)->loadObjectList();

		foreach($rows as $row)
		{
			$cat[] = $row->id_categories;
		}

		$cats = array_merge($cat, $this->getCatParents($cat));

		return $cats;
	}

	/**
	 * Given a starting array-set, this function returns the parent for every category.
	 *
	 * @param 	array $cat Categories we're looking for parents
	 * @return 	array Category parents
	 */
	public function getCatParents($cat)
	{
		$db = JFactory::getDbo();
		$cats = implode(',', $cat);

		$query = $db->getQuery(true)
					->select('cat_parent')
					->from('#__domus_categories')
					->where('id_categories IN ('.$cats.')');

		$rows = $db->setQuery($query)->loadObjectList();
		$return = array();
		foreach($rows as $row)
		{
			//exclude root parent
			if($row->cat_parent != 0) $return[] = $row->cat_parent;
		}

		if($return) $return = array_merge($return, $this->getCatParents($return));

		return $return;
	}

	public function getCatTree($published = true, $cats = array())
	{
		$whiteList = array();

		$db = JFactory::getDBO();

		$query = $db->getQuery(true);
		$query->select('*')
			  ->select('CASE WHEN CHAR_LENGTH(cat_alias) THEN CONCAT_WS(":", id_categories, cat_alias) ELSE id_categories END as catslug')
			  ->from('#__domus_categories')
			  ->order('cat_parent ASC, ordering ASC');

		if($published) $whiteList  = $this->getPublishedCats();
		if($cats)		$whiteList = array_merge($whiteList, $cats);
		if(count($whiteList)) $query->where('id_categories IN('.implode(',', $whiteList).')');

		$rows = $db->setQuery($query)->loadObjectList('id_categories');

		foreach($rows as $row)
		{
			$parent[$row->cat_parent][] = $row->id_categories;
		}

		return $this->createCatTree(0, $parent, $rows);
	}

	/**
	 * Method to return the list of published categories. To be here both the category
	 * AND the parent must be published.
	 *
	 * @return array Published categories id
	 */
	public function getPublishedCats()
	{
		static $publishedCats;

		$db = JFactory::getDbo();

		//let's cache the results
		if($publishedCats) return $publishedCats;

		$query = $db->getQuery(true);
		$query->select('a.id_categories')
				->from('#__domus_categories a')
				->leftJoin('#__domus_categories b ON (a.cat_parent = b.id_categories)')
				->where('a.cat_published = 1')
				->where('(b.cat_published = 1 OR b.cat_published IS NULL)');
		$publishedCats = $db->setQuery($query)->loadColumn();

		return $publishedCats;
	}

	public function createCatTree($parent, $parents, $rows)
	{
		$cats = $parents[$parent];

		if(!$cats) return array();

		foreach($cats as $cat)
		{
			$items[$cat] = $rows[$cat];
			$items[$cat]->children = $this->createCatTree($rows[$cat]->id_categories, $parents, $rows);
		}

		return $items;
	}

	/**
	 * Get category's children, at the moment gets only the first level.
	 * @param int $parent Category parent
	 * @TODO create a full recursive tree
	 */
	function catChildren($cat, $published = true)
	{
		$db = JFactory::getDbo();

		$query = $db->getQuery(true);
		$query->select('*')
			  ->select('CASE WHEN CHAR_LENGTH(cat_alias) THEN CONCAT_WS(":", id_categories, cat_alias) ELSE id_categories END as catslug')
			  ->from('#__domus_categories')
			  ->where('(id_categories = '.$cat.' OR cat_parent = '.$cat.')')
			  ->order('cat_parent ASC, ordering ASC');

		if($published) $query->where('id_categories IN('.implode(',', $this->getPublishedCats()).')');

		$rows = $db->setQuery($query)->loadObjectList();

		return $rows;
	}
}