<?php
/**
 * @package    	Domus Organizer
 * @subpackage 	Models
 * @author     	Davide Tampellini
 * @copyright 	Copyright (c)2011-2012 Davide Tampellini
 * @license 	GNU General Public License version 3, or later
 */

defined('_JEXEC') or die();

class DomusModelCustomers extends FOFModel
{
	public function buildQuery($overrideLimits = false)
	{
		$db = $this->getDbo();
		$query = FOFQueryAbstract::getNew($db)
					->select('cu.*, cg_descr, to_descr, st_descr')
					->from('#__domus_customers cu')
					->leftJoin('#__domus_customer_groups ON cu_id_customer_groups = id_customer_groups')
					->leftJoin('#__domus_towns ON cu_id_towns = id_towns')
					->leftJoin('#__domus_street_type ON cu_id_street_type = id_street_type');

		//mhm i'm asking for customers linked to a property
		$prop_owner = $this->getState('prop_owner', 0, 'int');
		if($prop_owner)
		{
			$query->innerJoin('#__domus_prop_owners ON pow_id_customers = id_customers')
				  ->where('pow_id_properties = '.$prop_owner);
		}

		//mhm i'm asking for customers not linked to a property
		$not_prop_owner = $this->getState('not_prop_owner', 0, 'int');
		if($not_prop_owner)
		{
			$query->leftJoin('#__domus_prop_owners ON pow_id_customers = id_customers')
				  ->where('(pow_id_properties != '.$not_prop_owner.' OR pow_id_properties IS NULL)');
		}

		$fulltext = $this->getState('query_string');
		if($fulltext)
		{
			$words = explode(' ', $fulltext);

			#AND search (+) and partial string (*)
			foreach($words as $string)
			{
				if(mb_strlen($string) > 3 ) $string = mb_substr($string, 0, mb_strlen($string) - 1);
				$search_string[] = '+'.$db->getEscaped($string).'*';
			}

			$query->where("(MATCH(cu_keywords) AGAINST ('".implode(' ', $search_string)."' IN BOOLEAN MODE))");
		}

		// Check the 'search type' field if we're really doing an advanced search
		if($this->getState('search_type', 0, 'int')){
		    $this->advancedSearch($query);
		}

		$order = $this->getState('filter_order', 'created_on', 'cmd');
		if(!in_array($order, array_keys($this->getTable()->getData()))) $order = 'created_on';

		$dir = $this->getState('filter_order_Dir', 'DESC', 'cmd');

		$query->order($order.' '.$dir);

		return $query;
	}

	public function &getItem($id = null)
	{
		if(!is_null($id))
		{
			$this->record = null;
			$this->setId($id);
		}

		if(empty($this->record))
		{
			$db = $this->getDbo();
			$query = FOFQueryAbstract::getNew($db);

			$query->select('cust.*, cg_descr, ty_name')
					->select('cat1.cat_name as cat1_name, cat2.cat_name as cat2_name, cat3.cat_name as cat3_name')
					->select('distr.di_descr as distr_res, distr_b.di_descr as distr_birth')
					->select('town.to_descr as town_res, town_b.to_descr as town_birth')
					->select('st_descr, so_descr')
					->select('us_add.us_name as add_name, us_add.us_surname as add_surname')
					->select('us_mod.us_name as mod_name, us_mod.us_surname as mod_surname')
					->from('#__domus_customers cust')
					->leftJoin('#__domus_types			 	 ON cu_req_id_types = id_types')
					->leftJoin('#__domus_customer_groups     ON cu_id_customer_groups = id_customer_groups')
					->leftJoin('#__domus_districts distr     ON cu_id_districts = distr.id_districts')
					->leftJoin('#__domus_districts distr_b 	 ON cu_district_birth = distr_b.id_districts')
					->leftJoin('#__domus_towns	 town    	 ON cu_id_towns = town.id_towns')
					->leftJoin('#__domus_towns     town_b  	 ON cu_town_birth = town_b.id_towns')
					->leftJoin('#__domus_street_type       	 ON cu_id_street_type = id_street_type')
					->leftJoin('#__domus_sources           	 ON cu_id_sources = id_sources')
					->leftJoin('#__domus_categories cat1	 ON cu_req_id_categories_1 = cat1.id_categories')
					->leftJoin('#__domus_categories cat2	 ON cu_req_id_categories_2 = cat2.id_categories')
					->leftJoin('#__domus_categories cat3	 ON cu_req_id_categories_3 = cat3.id_categories')
					->leftJoin('#__domus_users us_add		 ON created_by = us_add.us_id_joomla')
					->leftJoin('#__domus_users us_mod		 ON modified_by = us_mod.us_id_joomla')
					->where('id_customers = '.$this->id);
			$this->record = $db->setQuery($query)->loadObject();

			$this->onAfterGetItem($this->record);
		}

		return $this->record;
	}

	function onAfterSave($table)
	{
		$db = JFactory::getDbo();

		//require custom helper class
		require_once JPATH_ROOT.'/components/com_domus/helpers/model_helper.php';

		FBModelHelper::saveMultipleRows('Custtown', array('ct_id_customers' => $table->id_customers), array(), 'DomusTable');
		FBModelHelper::saveMultipleRows('Custroom', array('cr_id_customers' => $table->id_customers), array(), 'DomusTable');

		//clean up attrib table
		$query = $db->getQuery(true)
					->delete('#__domus_cust_requests')
					->where('req_id_customers = '.$table->id_customers);
		$db->setQuery($query)->query();

		$attribs = JRequest::getVar('req_id_attributes');

		$query = $db->getQuery(true)
					->insert('#__domus_cust_requests');

		foreach($attribs as $attrib)
		{
			if($attrib == '') continue;
			$query->values($table->id_customers.', '.$attrib);
		}

		//save customer request attributes
		if($query->values && count($query->values->getElements()))
		{
			$db->setQuery($query)->Query();
		}

		$keywords = $this->createKeywords($table->id_customers);

		$query = $db->getQuery(true)
					->update('#__domus_customers')
					->set('cu_keywords = '.$db->Quote($keywords))
					->where('id_customers = '.$table->id_customers);
		$db->setQuery($query)->Query();
	}

	public function batchCreateKeywords()
	{
		$db = JFactory::getDbo();

		$query = FOFQueryAbstract::getNew($db)
					->select('cust.*, cg_descr')
					->from('#__domus_customers cust')
					->leftJoin('#__domus_customer_groups ON id_customer_groups = cu_id_customer_groups');
		$rows = $db->setQuery($query)->loadObjectList();

		foreach($rows as $row)
		{
			$keywords = $this->createKeywords(0, $row);

			$query = FOFQueryAbstract::getNew($db)
						->update('#__domus_customers')
						->set('cu_keywords = '.$db->quote($keywords))
						->where('id_customers = '.$row->id_customers);
			$db->setQuery($query)->query();
		}

		return true;
	}

	public function createKeywords($id_customers = 0, $row = null)
	{
		require_once JPATH_ROOT.'/components/com_domus/helpers/html.php';

		$db 	  = JFactory::getDbo();
		$params	  = JComponentHelper::getParams('com_domus');
		$keywords = array();

		if(!$row)
		{
			$query = $db->getQuery(true)
						->select('cust.*, cg_descr')
						->from('#__domus_customers cust')
						->leftJoin('#__domus_customer_groups ON id_customer_groups = cu_id_customer_groups')
						->where('id_customers = '.$id_customers);
			$row = $db->setQuery($query)->loadObject();
		}

		if($params->get('surname'	, 1))	$keywords[] = $row->cu_name.' '.$row->cu_surname.' '.$row->cu_company_name;
		if($params->get('group'		, 1))	$keywords[] = $row->cg_descr;
		if($params->get('custType'	, 1)) 	$keywords[] = DomusHelperHtml::decoFix('cust_type', $row->cu_type);

		return implode(' ', $keywords);
	}

	private function advancedSearch($query)
	{
		require_once JPATH_ROOT.'/components/com_domus/helpers/format.php';

		$db = JFactory::getDbo();

		$type_search = $this->getState('type_search', 0, 'int');
		if($type_search)    $query->where('cu_req_id_types = '.$type_search);

		$cat_search = $this->getState('cat_search', 0, 'int');
		if($cat_search)
		{
            $query->where('(cu_req_id_categories_1 = '.$cat_search.' OR
                            cu_req_id_categories_2 = '.$cat_search.' OR
                            cu_req_id_categories_3 = '.$cat_search.')');
		}

		$price_min = DomusHelperFormat::priceToDatabase($this->getState('price_min', 0, 'raw'));
		if($price_min)    $query->where('cu_req_max_price >= '.$price_min);

		$price_max = DomusHelperFormat::priceToDatabase($this->getState('price_max', 0, 'raw'));
		if($price_max)    $query->where('cu_req_max_price <= '.$price_max);

		$sup_min = $this->getState('sup_min', 0, 'int');
		if($sup_min)      $query->where('cu_req_min_space >= '.$sup_min);

		$sup_max = $this->getState('sup_max', 100000, 'int');
		if($sup_max)      $query->where('cu_req_min_space <= '.$sup_max);

		$floor	 = $this->getState('floor_search', null, 'int');
		if($floor !== null)	$query->where('cu_req_max_floor >= '.$floor);

		$max_units = $this->getState('max_units', 0, 'int');
		if($max_units) 		$query->where('cu_req_max_units >= '.$max_units);

		$user_add = $this->getState('created_by', 0, 'int');
		if($user_add)	$query->where('cu.created_by = '.$user_add);

		$attribs = FOFModel::getTmpInstance('Attributes', 'DomusModel')->getList(true, 'att_type');
		$attribs = array_keys($attribs);

		foreach($attribs as $attrib)
		{
			$value = $this->getState($attrib.'_SEARCH', 0, 'int');
			if($value)	$attrib_search[] = $value;
		}

		// Filter property attributes (it's a nasty one)
		if($attrib_search)
		{
			$subquery = FOFQueryAbstract::getNew()
							->select('req_id_customers')
							->from('#__domus_cust_requests')
							->where('req_id_attributes IN ('.implode(',', $attrib_search).')')
							->group('req_id_customers')
							->having('COUNT(*) = '.count($attrib_search));
			$sub_cust = $db->setQuery($subquery)->loadColumn();
			$query->where('id_customers IN('.implode(',', $sub_cust).')');
		}

		$distrs = $this->getState('districts_search', array(), 'array');
		$towns  = $this->getState('towns_search'	, array(), 'array');
		$zones  = $this->getState('zones_search'	, array(), 'array');
		$szones = $this->getState('szones_search'	, array(), 'array');

		for($i = 0; $i < count($distrs); $i++)
		{
			$position = 'ct_id_districts = '.$distrs[$i];

			if($towns[$i])	$position .= ' AND ct_id_towns = '.$towns[$i];
			if($zones[$i])	$position .= ' AND ct_id_zones = '.$zones[$i];
			if($szones[$i])	$position .= ' AND ct_id_szones = '.$szones[$i];

			$positions[] = $position;
		}

		if($positions)
		{
			$query->innerJoin('#__domus_cust_towns ON ct_id_customers = id_customers');
			$query->where('('.implode(' OR ', $positions).')');
			$query->group('id_customers');
		}

		$wcs = $this->getState('wc_search', 0, 'int');
		if($wcs)
		{
			$subquery = FOFQueryAbstract::getNew()
							->select('cr_id_customers')
							->from('#__domus_cust_rooms')
							->innerJoin('#__domus_rooms ON cr_id_rooms = id_rooms')
							->where('ro_group = '.$db->Quote('BATHROOM'))
							->group('cr_id_customers')
							->having('SUM(cr_quantity) <= '.$wcs);
			$sub_cust = $db->setQuery($subquery)->loadColumn();
			$query->where('id_customers IN ('.implode(',', $sub_cust).')');
		}

		$beds = $this->getState('bed_search', 0, 'int');
		if($beds)
		{
			$subquery = FOFQueryAbstract::getNew()
							->select('cr_id_customers')
							->from('#__domus_cust_rooms')
							->innerJoin('#__domus_rooms ON cr_id_rooms = id_rooms')
							->where('ro_group = '.$db->Quote('BEDROOM'))
							->group('cr_id_customers')
							->having('SUM(cr_quantity) <= '.$beds);
			$sub_cust = $db->setQuery($subquery)->loadColumn();
			$query->where('id_customers IN ('.implode(',', $sub_cust).')');
		}

		$rooms 	  = FOFInput::getArray('room_type', array(), $this->input);
		$quantity = FOFInput::getArray('room_quantity', array(), $this->input);

		if($rooms && $quantity)
		{
			$sub_cust = array();
			for($i = 0; $i < count($rooms); $i++)
			{
				if(!$quantity[$i]) continue;

				$subquery = FOFQueryAbstract::getNew()
								->select('cr_id_customers')
								->from('#__domus_cust_rooms')
								->where('cr_id_rooms = '.$rooms[$i])
								->group('cr_id_customers')
								->having('SUM(cr_quantity) <= '.$quantity[$i]);
				$t_cust = $db->setQuery($subquery)->loadColumn();

				if($sub_cust){
					$sub_cust = array_intersect($sub_cust, $t_cust);
				}
				else{
					$sub_cust = $t_cust;
				}
			}

			$sub_cust[] = -1;
			$query->where('id_customers IN('.implode(',', $sub_cust).')');
		}
	}
}