<?php
/**
 * @package    	Domus Organizer
 * @subpackage 	Models
 * @author     	Davide Tampellini
 * @copyright 	Copyright (c)2011-2012 Davide Tampellini
 * @license 	GNU General Public License version 3, or later
 */

defined('_JEXEC') or die();

class DomusModelProperties extends FOFModel
{
	private $alreadyFiltered = false;

	public function buildQuery($overrideLimits = false)
	{
		$db = $this->getDbo();
		$query = FOFQueryAbstract::getNew($db)
					->select('prop.*, cat_name, ty_name, to_descr, st_descr')
					->select('cu_company_name, cu_name, cu_surname')
					->select('CASE WHEN CHAR_LENGTH(pr_alias) THEN CONCAT_WS(":", id_properties, pr_alias) ELSE id_properties END as slug')
					->select('CASE WHEN CHAR_LENGTH(cat_alias) THEN CONCAT_WS(":", id_categories, cat_alias) ELSE id_categories END as catslug')
					->from('#__domus_properties prop')
					->innerJoin('#__domus_types ON id_types = pr_id_types')
					->leftJoin('#__domus_categories ON id_categories = pr_id_categories')
					->leftJoin('#__domus_towns ON id_towns = pr_id_towns')
					->leftJoin('#__domus_street_type ON id_street_type = pr_id_street_type');

		$this->internalSearch($query);
		$this->publicSearch($query);

		// Check the 'search type' field if we're really doing an advanced search
		if($this->getState('search_type', 0, 'int')){
		    $this->advancedSearch($query);
		}

		$order = $this->getState('filter_order', 'created_on', 'cmd');
		$allowed_order = array_keys($this->getTable()->getData());
		$allowed_order[] = 'price';
		if(!in_array($order, $allowed_order)) $order = 'created_on';

		$dir = $this->getState('filter_order_Dir', 'DESC', 'cmd');

		// public order price, private treaty properties should be always on the bottom
		// (users are not allowed to guess their price)
		if($order == 'price')
		{
			$query->order('pr_private_treaty ASC, pr_price '.$dir);
		}
		else $query->order($order.' '.$dir);

		return $query;
	}

	public function buildCountQuery()
	{
		// Resets the flag for public properties only
		$this->alreadyFiltered = false;
		return false;
	}

	public function &getItem($id = null)
	{
		if(!is_null($id))
		{
			$this->record = null;
			$this->setId($id);
		}

		if(empty($this->record))
		{
			$db = $this->getDbo();
			$query = FOFQueryAbstract::getNew($db);

			$query->select('propr.*, ty_name, cat_name, st_descr, to_descr, di_descr')
					->select('CASE WHEN CHAR_LENGTH(pr_alias) THEN CONCAT_WS(":", id_properties, pr_alias) ELSE id_properties END as slug')
					->select('CASE WHEN CHAR_LENGTH(cat_alias) THEN CONCAT_WS(":", id_categories, cat_alias) ELSE id_categories END as catslug')
					->select('zo_descr, sz_descr')
					->select('us_add.us_name as add_name, us_add.us_surname as add_surname')
					->select('us_mod.us_name as mod_name, us_mod.us_surname as mod_surname')
					->from('#__domus_properties propr')
					->where('id_properties = '.$this->id)
					->innerJoin('#__domus_types ON id_types = pr_id_types')
					->innerJoin('#__domus_categories ON id_categories = pr_id_categories')
					->innerJoin('#__domus_towns ON pr_id_towns = id_towns')
					->innerJoin('#__domus_districts ON pr_id_districts = id_districts')
					->leftJoin ('#__domus_street_type ON pr_id_street_type = id_street_type')
					->leftJoin ('#__domus_zones ON pr_id_zones = id_zones')
					->leftJoin ('#__domus_szones ON pr_id_szones = id_szones')
					->leftJoin ('#__domus_users us_add	ON created_by = us_add.us_id_joomla')
					->leftJoin ('#__domus_users us_mod	ON modified_by = us_mod.us_id_joomla');

			$enabled = $this->getState('enabled', 0 ,'int');
			if($enabled) $query->where('propr.enabled = 1');

			$this->record = $db->setQuery($query)->loadObject();

			$this->onAfterGetItem($this->record);
		}

		return $this->record;
	}

	function onAfterSave($table)
	{
		$db = JFactory::getDBO();

		//require custom helper class
		require_once JPATH_ROOT.'/components/com_domus/helpers/model_helper.php';
		FBModelHelper::saveMultipleRows('Proproom', array('pro_id_properties' => $table->id_properties), array(), 'DomusTable');
		FBModelHelper::saveMultipleRows('Image', array('img_id_properties' => $table->id_properties), array('img_published'), 'DomusTable');

		//clean up attrib table
		$query = $db->getQuery(true)
					->delete('#__domus_prop_attributes')
					->where('patt_id_properties = '.$table->id_properties);
		$db->setQuery($query)->Query();

		$attribs = JRequest::getVar('patt_id_attributes');

		$query = $db->getQuery(true)
					->insert('#__domus_prop_attributes');

		foreach($attribs as $attrib)
		{
			if($attrib == '') continue;
			$query->values($table->id_properties.', '.$attrib);
		}

		//save property attributes
		if($query->values && count($query->values->getElements()))
		{
			$db->setQuery($query)->Query();
		}

		//recreate keywords
		$keywords = $this->createKeywords($table->id_properties);

		//save room count
		$query = $db->getQuery(true)
					->select('COUNT(*) as count, ro_group')
					->from('#__domus_rooms')
					->innerJoin('#__domus_prop_rooms ON pro_id_rooms = id_rooms')
					->where('pro_id_properties = '.$table->id_properties)
					->group('ro_group');
		$rooms = $db->setQuery($query)->loadObjectList('ro_group');

		foreach($rooms as $room)
		{
			$total += $room->count;
		}

		//create metadesc field (only if there isn't)
		if (!$this->row->pr_metadesc)
		{
			$query = $db->getQuery(true)
						->select('to_descr, di_descr, cat_name, ty_name')
						->from('#__domus_properties')
						->innerJoin('#__domus_towns ON pr_id_towns = id_towns')
						->innerJoin('#__domus_districts ON pr_id_districts = id_districts')
						->innerJoin('#__domus_categories ON pr_id_categories = id_categories')
						->innerJoin('#__domus_types ON pr_id_types = id_types')
						->where('id_properties = '.$table->id_properties);
			$row = $db->setQuery($query)->loadObject();

			$metadesc  = $row->to_descr.' ('.$row->di_descr.') '.$row->cat_name.' - '.$row->ty_name.' ';
			if($this->row->pr_descr) $metadesc .= substr($this->row->pr_descr, 0, 100).'...';
		}

		//save everything
		$query = $db->getQuery(true)
					->update('#__domus_properties')
					->set("pr_keywords = '".$keywords."'")
					->set("pr_bedrooms = ".(int)$rooms['BEDROOM']->count)
					->set("pr_wc = ".(int)$rooms['BATHROOM']->count)
					->set("pr_rooms = ".(int) $total)
					->where("id_properties = ".$table->id_properties);

		if($metadesc)	$query->set("pr_metadesc = ".$db->Quote($metadesc));

		$db->setQuery($query)->Query();

		//save images
		$this->saveImages($table->id_properties);
	}

	public function copy()
	{
		$db = JFactory::getDbo();

		$this->setIDsFromRequest();
		$ids   = $this->getIds();
		$prop  = $this->getTable($this->table);

		$tblImage = FOFTable::getAnInstance('Image', 'DomusTable');
		$tblRoom  = FOFTable::getAnInstance('Proproom', 'DomusTable');

		foreach($ids as $id_prop)
		{
			$prop->load($id_prop);

			$prop->id_properties	 = null;
			$prop->pr_obj_id 	    .= '-copy';
			$prop->pr_title			.= '-copy';
			$prop->pr_alias	   		.= '-copy';
			$prop->created_on		 = null;
			$prop->modified_on		 = null;
			$prop->created_by		 = null;
			$prop->modified_by		 = null;
			$prop->enabled			 = 0;

			$prop->store();

			//copy attributes
			$query = $db->getQuery(true)
						->select('patt_id_attributes')
						->from('#__domus_prop_attributes')
						->where('patt_id_properties = '.$id_prop);
			$attribs = $db->setQuery($query)->loadColumn();

			if($attribs)
			{
				$query = $db->getQuery(true)
							->insert('#__domus_prop_attributes');
				foreach($attribs as $attrib)
				{
					$query->values($prop->id_properties.', '.$attrib);
				}

				if($query->values && count($query->values->getElements())) $db->setQuery($query)->Query();
			}

			//copy owners
			$query = $db->getQuery(true)
						->select('pow_id_customers')
						->from('#__domus_prop_owners')
						->where('pow_id_properties = '.$id_prop);
			$owners = $db->setQuery($query)->loadColumn();

			if($owners)
			{
				$query = $db->getQuery(true)
							->insert('#__domus_prop_owners');
				foreach($owners as $owner)
				{
					$query->values($prop->id_properties.', '.$owner);
				}

				if($query->values && count($query->values->getElements())) $db->setQuery($query)->Query();
			}

			//copy rooms
			$query = $db->getQuery(true)
						->select('id_prop_rooms')
						->from('#__domus_prop_rooms')
						->where('pro_id_properties = '.$id_prop);
			$rooms = $db->setQuery($query)->loadColumn();

			foreach($rooms as $room)
			{
				$tblRoom->load($room);

				$tblRoom->id_prop_rooms	    = null;
				$tblRoom->pro_id_properties = $prop->id_properties;

				$tblRoom->store();
			}

			//copy images
			$query = $db->getQuery(true)
						->select('id_images')
						->from('#__domus_images')
						->where('img_id_properties = '.$id_prop);
			$images = $db->setQuery($query)->loadColumn();

			foreach($images as $image)
			{
				$tblImage->load($image);

				$tblImage->id_images 		 = null;
				$tblImage->img_id_properties = $prop->id_properties;

				$tblImage->store();
			}

		}

		return true;
	}

	public function batchCreateKeywords()
	{
		$db = JFactory::getDbo();

		$query = FOFQueryAbstract::getNew($db)
				  ->select('id_properties, pr_title, pr_obj_id, pr_street, pr_domus_published, cat_name, to_descr, st_descr, ty_name')
				  ->from('#__domus_properties')
				  ->leftJoin('#__domus_categories ON pr_id_categories = id_categories')
				  ->leftJoin('#__domus_towns ON pr_id_towns = id_towns')
				  ->leftJoin('#__domus_street_type ON pr_id_street_type = id_street_type')
				  ->leftJoin('#__domus_types ON pr_id_types = id_types');

		$rows = $db->setQuery($query)->loadObjectList();

		foreach($rows as $row)
		{
			$keywords = $this->createKeywords(0, $row);

			$query = FOFQueryAbstract::getNew($db)
						->update('#__domus_properties')
						->set('pr_keywords = '.$db->quote($keywords))
						->where('id_properties = '.$row->id_properties);
			$db->setQuery($query)->query();
		}

		return true;
	}

	public function createKeywords($id_property, $row = null)
	{
		require_once JPATH_ROOT.'/components/com_domus/helpers/html.php';

		$db		  = JFactory::getDbo();
		$params   = JComponentHelper::getParams('com_domus');
		$keywords = array();

		if(!$row)
		{
			$query = $db->getQuery(true);
			$query->select('id_properties, pr_title, pr_obj_id, pr_street, pr_domus_published, cat_name, to_descr, st_descr, ty_name')
				  ->from('#__domus_properties')
				  ->leftJoin('#__domus_categories ON pr_id_categories = id_categories')
				  ->leftJoin('#__domus_towns ON pr_id_towns = id_towns')
				  ->leftJoin('#__domus_street_type ON pr_id_street_type = id_street_type')
				  ->leftJoin('#__domus_types ON pr_id_types = id_types')
				  ->where('id_properties = '.$id_property);

			$row = $db->setQuery($query)->loadObject();
		}

		if($params->get('title', 1))		$keywords[] = $row->pr_title;
		if($params->get('category', 1))		$keywords[] = $row->cat_name;
		if($params->get('town', 1)) 		$keywords[] = $row->to_descr;
		if($params->get('address', 1))		$keywords[] = $row->st_descr.' '.$row->pr_street;
		if($params->get('proprType', 1))	$keywords[] = $row->ty_name;
		if($params->get('objid', 1))		$keywords[] = $row->pr_obj_id;
		$keywords[] = DomusHelperHtml::decoFix('domus_published', $row->pr_domus_published);

		if($params->get('owners', 1))
		{
			$query = $db->getQuery(true);
			$query->select('cu_name, cu_surname, cu_company_name')
				  ->from('#__domus_customers')
				  ->innerJoin('#__domus_prop_owners ON pow_id_customers = id_customers')
				  ->where('pow_id_properties = '.$row->id_properties);
			$owners = $db->setQuery($query)->loadObjectList();

			foreach($owners as $own)
			{
				$keywords[] = $own->cu_name.' '.$own->cu_surname.' '.$own->cu_company_name;
			}
		}

		return implode(' ', $keywords);
	}

	public function saveImages($id_property)
	{
		$images = JRequest::getVar('prop_images', '', 'FILES');

		//no images to save
		if(!$images['name'][0])  return;

		require_once JPATH_ROOT.'/components/com_domus/helpers/gallery.php';

		$params    	= JComponentHelper::getParams('com_domus');

		$imagePath 	= JPATH_ROOT.'/'.$params->get('galleryPath', 'media/com_domus/photos/');
		if(substr($imagePath, -1, 1) != '/') $imagePath .= '/';

		$pWidth	   	= $params->get('photoWidth'		, 640);
		$pHeight	= $params->get('photoHeight'	, 480);
		$pQuality	= $params->get('photoQuality'	, 85);

		$tWidth	   	= $params->get('thumbWidth'		, 160);
		$tHeight	= $params->get('thumbHeight'	, 120);
		$tQuality	= $params->get('thumbQuality'	, 80);

		if(substr($imagePath, -1, 1) != '/') $imagePath .= '/';

		$destFolder = $imagePath.$id_property.'/';
		$buffer = '<!DOCTYPE html><title></title>';

		//create general photo folder
		if(!JFolder::exists($imagePath.'/index.html'))	JFile::write($imagePath.'/index.html', $buffer);

		//create property folder
		if(!JFolder::exists($destFolder))JFile::write($destFolder.'index.html', $buffer);
		if(!JFolder::exists($destFolder.'/thumbs')) JFile::write($destFolder.'thumbs/index.html', $buffer);

		for($i = 0; $i < count($images['name']); $i++)
		{
			$microsec = str_replace('0.', '', explode(' ', microtime()));
			//sanitize image name
			$name = DomusHelperGallery::filename_safe(JFile::stripExt($images['name'][$i])).'_'.$microsec[0];

			$rc_photo = DomusHelperGallery::resizeIMG(0, $images['tmp_name'][$i], $destFolder.$name.'.jpg', $pWidth, $pHeight, 1, $pQuality);
			$rc_thumb = DomusHelperGallery::resizeIMG(1, $images['tmp_name'][$i], $destFolder.'thumbs/t_'.$name.'.jpg', $tWidth, $tHeight, 1, $tQuality);

			if($rc_photo && $rc_thumb)	DomusHelperGallery::storeImage($id_property, $destFolder.$name.'.jpg');
		}
	}

	public function getUserStateFromRequest($key, $request, $default = null, $type = 'none')
	{
		return parent::getUserStateFromRequest($key, $request, $default, $type);
	}

	private function internalSearch($query)
	{
		$db = JFactory::getDbo();

		//mhm i'm asking for properties linked to a customer
		$prop_owner = $this->getState('prop_owner', 0, 'int');
		if($prop_owner)
		{
			$query->innerJoin('#__domus_prop_owners ON pow_id_properties = id_properties')
				  ->innerJoin('#__domus_customers ON pow_id_customers = id_customers')
				  ->where('pow_id_customers = '.$prop_owner);
		}

		//mhm i'm asking for properties not linked to a customer. I have to fetch properties
		//not already linked with the current customer
		$not_prop_owner = $this->getState('not_prop_owner', 0, 'int');
		if($not_prop_owner)
		{
			//since I can have properties with multiple owners, I have to exclude them
			//using the property id.
			$sql = FOFQueryAbstract::getNew($db)
					->select('pow_id_properties')
					->from('#__domus_prop_owners')
					->where('pow_id_customers = '.$not_prop_owner);
			$not_customers = $db->setQuery($sql)->loadColumn();
			$not_customers[] = -1;

			$query->leftJoin('#__domus_prop_owners ON pow_id_properties = id_properties')
				  ->leftJoin('#__domus_customers ON pow_id_customers = id_customers')
				  ->where('id_properties NOT IN('.implode(', ', $not_customers).')');
		}

		//i didn't ask for anything special (simple list view) let's join them with customers
		//to get the first owner to show in the property list
		if(!$prop_owner && !$not_prop_owner)
		{
			$query->leftJoin('#__domus_prop_owners ON pow_id_properties = id_properties')
				  ->leftJoin('#__domus_customers ON pow_id_customers = id_customers');
		}

		//join to the customers table. I need the group clause since there could be more than
		//an owner on the same property.
		//P.A. the join with the owner table could be a left or a inner one (up here)
		$query->group('id_properties');

		$fulltext = $this->getState('query_string');
		if($fulltext)
		{
			$words = explode(' ', $fulltext);

			//AND search (+) with patial string (*)
			foreach($words as $string)
			{
				if(mb_strlen($string) > 3 ) $string = mb_substr($string, 0, mb_strlen($string) - 1);
				$search_string[] = '+'.$db->getEscaped($string).'*';
			}

			$query->where("MATCH(pr_keywords) AGAINST ('".implode(' ', $search_string)."' IN BOOLEAN MODE)");
		}

		$int_publish = $this->getState('internal_publish', 0, 'int');
		if($int_publish === 0)
		{
			$query->where('pr_domus_published = 1');
		}
	}

	private function publicSearch($query)
	{
		//published properties
		$enabled = $this->getState('enabled', null, 'int');
		if($enabled !== null)	$query->where('prop.enabled = '.$enabled);

		//public properties only (property, category and type published)
		$pub_published = $this->getState('pub_published', 0, 'int');
		if($pub_published)
		{
			$this->filterPublished($query);
			$pub_cats = FOFModel::getTmpInstance('Categories', 'DomusModel')
						->getPublishedCats();

			$query->where('pr_id_categories IN ('.implode(', ', $pub_cats).')');
		}

		//properties from a parent cat (gets properties in children cats, too)
		$parent_cat = $this->getState('parent_cat', 0, 'int');
		if($parent_cat)
		{
			$this->filterPublished($query);

			$cats = FOFModel::getTmpInstance('Categories', 'DomusModel')
							->catChildren($parent_cat);

			foreach($cats as $cat)
			{
				$parent_cats[] = $cat->id_categories;
			}

			$query->where('pr_id_categories IN ('.implode(', ', $parent_cats).')');
		}

		//hot/normal properties only
		$hot = $this->getState('pr_hot', null, 'int');
		if($hot !== null)		$query->where('pr_hot = '.$hot);

		//params from public search
		$cat_search = $this->getState('public_cat_search', 0, 'int');
		if($cat_search)
		{
			$this->filterPublished($query);

			$cats = FOFModel::getTmpInstance('Categories', 'DomusModel')
							->catChildren($cat_search);

			foreach($cats as $cat)
			{
				$search_cats[] = $cat->id_categories;
			}

			$query->where('pr_id_categories IN ('.implode(', ', $search_cats).')');
		}

		$distr_search = $this->getState('distr_search', 0, 'int');
		if($distr_search)		$query->where('pr_id_districts = '.$distr_search);

		$town_search = $this->getState('town_search', 0, 'int');
		if($town_search)		$query->where('pr_id_towns = '.$town_search);

		$zone_search = $this->getState('zone_search', 0, 'int');
		if($zone_search)		$query->where('pr_id_zones = '.$zone_search);

		$szone_search = $this->getState('szone_search', 0, 'int');
		if($szone_search)		$query->where('pr_id_szones = '.$szone_search);

		$type_search = $this->getState('type_search', 0, 'int');
		if($type_search)		$query->where('pr_id_types = '.$type_search);

		$price_search = $this->getState('price_search', 0, 'int');
		if($price_search)
		{
			require_once JPATH_ROOT.'/components/com_domus/helpers/property.php';
			$query->where(DomusHelperProperty::buildQueryPrice($price_search));
		}
	}

	private function advancedSearch($query)
	{
	    require_once JPATH_ROOT.'/components/com_domus/helpers/format.php';

	    $db = JFactory::getDbo();

        $type = $this->getState('type_search', 0);
        if($type)    $query->where('pr_id_types = '.$type);

        $cat_search = $this->getState('cat_search', 0);
        if($cat_search)
        {
            $cats = FOFModel::getTmpInstance('Categories', 'DomusModel')
							->catChildren($cat_search, false);

			foreach($cats as $cat)
			{
				$search_cats[] = $cat->id_categories;
			}

			$query->where('pr_id_categories IN ('.implode(', ', $search_cats).')');
        }

        $price_min = DomusHelperFormat::priceToDatabase($this->getState('price_min', 0));
        if($price_min)    $query->where('pr_price >= '.$price_min);

        $price_max = DomusHelperFormat::priceToDatabase($this->getState('price_max', 0));
        if($price_max)    $query->where('pr_price <= '.$price_max);

        $sup_min   = $this->getState('sup_min', 0);
        if($sup_min)      $query->where('pr_space >= '.$sup_min);

        $sup_max   = $this->getState('sup_max', 0);
        if($sup_max)      $query->where('pr_space <= '.$sup_max);

        $max_floor = $this->getState('floor_search', 0);
        if($max_floor)    $query->where('pr_floor <= '.$max_floor);

        $max_units = $this->getState('max_units', 0);
        if($max_units)    $query->where('pr_units <= '.$max_units);

        $user      = $this->getState('user_search', 0);
        if($user)         $query->where('prop.created_by = '.$user);

		$attribs = FOFModel::getTmpInstance('Attributes', 'DomusModel')->getList(true, 'att_type');
		$attribs = array_keys($attribs);

		foreach($attribs as $attrib)
		{
			$value = $this->getState($attrib.'_SEARCH', 0, 'int');
			if($value)	$attrib_search[] = $value;
		}

		// Filter property attributes (it's a nasty one)
		if($attrib_search)
		{
			$subquery = FOFQueryAbstract::getNew()
							->select('patt_id_properties')
							->from('#__domus_prop_attributes')
							->where('patt_id_attributes IN ('.implode(',', $attrib_search).')')
							->group('patt_id_properties')
							->having('COUNT(*) = '.count($attrib_search));
			$sub_prop = $db->setQuery($subquery)->loadColumn();
			$query->where('id_properties IN('.implode(',', $sub_prop).')');
		}

		$distrs = $this->getState('districts_search', array(), 'array');
		$towns  = $this->getState('towns_search'	, array(), 'array');
		$zones  = $this->getState('zones_search'	, array(), 'array');
		$szones = $this->getState('szones_search'	, array(), 'array');

		for($i = 0; $i < count($distrs); $i++)
		{
			$position = 'pr_id_districts = '.$distrs[$i];

			if($towns[$i])	$position .= ' AND pr_id_towns = '.$towns[$i];
			if($zones[$i])	$position .= ' AND pr_id_zones = '.$zones[$i];
			if($szones[$i])	$position .= ' AND pr_id_szones = '.$szones[$i];

			$positions[] = $position;
		}

		if($positions) $query->where('('.implode(' OR ', $positions).')');

		$wcs = $this->getState('wc_search', 0, 'int');
		if($wcs)	$query->where('pr_wc >= '.$wcs);

		$beds = $this->getState('bed_search', 0, 'int');
		if($beds)	$query->where('pr_bedrooms >= '.$beds);

		$rooms 	  = FOFInput::getArray('room_type', array(), $this->input);
		$quantity = FOFInput::getArray('room_quantity', array(), $this->input);

		if($rooms && $quantity)
		{
			for($i = 0; $i < count($rooms); $i++)
			{
				if(!$quantity[$i]) continue;
				$subquery = FOFQueryAbstract::getNew()
							   ->select('pro_id_properties')
							   ->from('#__domus_prop_rooms')
							   ->where('pro_id_rooms = '.$rooms[$i])
							   ->group('pro_id_properties')
							   ->having('COUNT(*) >= '.$quantity[$i]);
				$t_prop = $db->setQuery($subquery)->loadColumn();

				if($white_prop){
					$white_prop = array_intersect($white_prop, $t_prop);
				}
				else{
					$white_prop = $t_prop;
				}
			}

			$white_prop[] = -1;
			$query->where('id_properties IN('.implode(',', $white_prop).')');
		}
	}

	/**
	 * Adds where clause to fetch only published properties.
	 *
	 * @param FOFQueryAbstract | JDatabaseQuery $query
	 */
	private function filterPublished($query)
	{
		if($this->alreadyFiltered)	return;

		$query->where('prop.enabled = 1')
			  ->where('ty_published = 1');

		$this->alreadyFiltered = true;
	}
}