<?php
/**
 * @package    	Domus Organizer
 * @author     	Davide Tampellini
 * @copyright 	Copyright (c)2011-2012 Davide Tampellini
 * @license 	GNU General Public License version 3, or later
 */

defined('_JEXEC') or die();

function domusBuildRoute(&$query)
{

	//if(function_exists('xdebug_break')) xdebug_break();

	$do_views = array('public', 'publics');
	$segments = array();

	//first of all let's check if the the link belongs to another menu
	//only do that if I have enough information (menu items only have option & itemid)
	if(isset($query['view']))
	{
		$qsearch = $query;
		if(isset($qsearch['Itemid'])) unset($qsearch['Itemid']);

		$menuid = DomusRouterHelper::findMenu($qsearch)->id;

		if($menuid) $query['Itemid'] = $menuid;
	}

	//Let's do the routing logic only if I'm in the public area (I don't care having nice SEF url in the admin one)
	if(!in_array($query['view'], $do_views)) return array();
	if($query['layout'] == 'contact')		 return array();

	if($query['layout'] == 'property_print')
	{
		$segments[] = 'print';
		unset($query['layout']);
	}

	if(isset($query['catid']))
	{
		$segments[] = $query['catid'];
		unset( $query['catid'] );
	};

	if(isset($query['id']))
	{
		$segments[] = $query['id'];
		unset( $query['id'] );
	};

	//unset layout only if it's a standard one
	if($query['layout'] != 'search') unset($query['layout']);

	unset($query['view']);
	unset($query['c']);
	unset($query['task']);

	unset($query['tmpl']);

	return $segments;
}

function domusParseRoute(&$segments)
{
	$vars = array();
	$app  = JFactory::getApplication();
	$menu = $app->getMenu();
	$item = $menu->getActive();

	//Count segments
	$count = count( $segments );

	//Handle View and Identifier
	switch( $item->query['layout'] )
	{
		case 'categories':
		default:
			//looking for a single category
			if($count == 1)
			{
				$vars['view']   = 'publics';
				$vars['layout'] = 'category	';
			}

			//looking for a single property
			if($count == 2)
			{
				$vars['view']   = 'publics';
				$vars['layout'] = 'property';
			}

			//print template
			if($count == 3)
			{
				$vars['view']	= 'publics';
				$vars['layout'] = 'property_print';
				$vars['tmpl'] 	= 'component';
			}
			$id = explode( ':', $segments[$count-1] );
			$vars['id'] = (int) $id[0];
			break;
		case 'category':
			$id   = explode( ':', $segments[$count-1] );
			$vars['id']   = (int) $id[0];

			break;
       }

	return $vars;
}

class DomusRouterHelper
{
	/**
	 * Finds a menu whose query parameters match those in $qoptions
	 * @param array $qoptions The query parameters to look for
	 * @param array $params The menu parameters to look for
	 * @return null|object Null if not found, or the menu item if we did find it
	 */
	static public function findMenu($qoptions = array(), $params = null)
	{
		// Convert $qoptions to an object
		if(empty($qoptions) || !is_array($qoptions)) $qoptions = array();

		$menus 	  = JMenu::getInstance('site');
		$menuitem = $menus->getActive();

		// First check the current menu item (fastest shortcut!)
		if(is_object($menuitem)) {
			if(self::checkMenu($menuitem, $qoptions, $params)) {
				return $menuitem;
			}
		}

		foreach($menus->getMenu() as $item)
		{
			if(self::checkMenu($item, $qoptions, $params)) return $item;
		}

		return null;
	}

	/**
	 * Checks if a menu item conforms to the query options and parameters specified
	 *
	 * @param object $menu A menu item
	 * @param array $qoptions The query options to look for
	 * @param array $params The menu parameters to look for
	 * @return bool
	 */
	static public function checkMenu($menu, $qoptions, $params = null)
	{
		$query = $menu->query;
		foreach($qoptions as $key => $value)
		{
			if(is_null($value)) continue;
			if(!isset($query[$key])) return false;
			if($query[$key] != $value) return false;
		}

		if(!is_null($params))
		{
			$menus = JMenu::getInstance('site');
			$check =  $menu->params instanceof JRegistry ? $menu->params : $menus->getParams($menu->id);

			foreach($params as $key => $value)
			{
				if(is_null($value)) continue;
				if( $check->get($key) != $value ) return false;
			}
		}

		return true;
	}
}