<?php
/**
 * @package    	Domus Organizer
 * @author     	Davide Tampellini
 * @copyright 	Copyright (c)2011-2012 Davide Tampellini
 * @license 	GNU General Public License version 3, or later
 */

defined('_JEXEC') or die();

class Com_DomusInstallerScript
{
	/** @var string The component's name */
	protected $_fabbrica_extension = 'com_domus';

	protected $installation_queue = array(
					// modules => { (folder) => { (module) => { (position), (published) } }* }*
					'modules' => array(
						'admin' => array(
						),
						'site' => array(
							'mod_domussearch' => array('left', 0)
						)
					),
					// plugins => { (folder) => { (element) => (published) }* }*
				);

	protected $status;

	/**
	 * Joomla! pre-flight event
	 *
	 * @param string $type Installation type (install, update, discover_install)
	 * @param JInstaller $parent Parent object
	 */
	public function preflight($type, $parent)
	{
		// Bugfix for "Can not build admin menus"
		if(in_array($type, array('install','discover_install'))) {
			$this->_bugfixDBFunctionReturnedNoError();
		} else {
			$this->_bugfixCantBuildAdminMenus();
		}

		// Only allow to install on Joomla! 2.5.0 or later
		return version_compare(JVERSION, '2.5.0', 'ge');
	}

	/**
	 * Runs after install, update or discover_update
	 * @param string $type install, update or discover_update
	 * @param JInstaller $parent
	 */
	function postflight( $type, $parent )
	{
		$this->src 	 = $parent->getParent()->getPath('source');
		$fofStatus = $this->_installFOF($parent);
		$this->installModules();

		require_once JPATH_LIBRARIES.'/fof/include.php';

		$utilities = FOFModel::getTmpInstance('Utilities', 'DomusModel');
		$utilities->rebuildCss();

		// Show the post-installation page
		$this->_renderPostInstallation($status, $fofStatus, $parent);
	}

	protected function installModules()
	{
		if(count($this->installation_queue['modules'])) {
			foreach($this->installation_queue['modules'] as $folder => $modules) {
				if(count($modules)) foreach($modules as $module => $modulePreferences) {
					// Install the module
					if(empty($folder)) $folder = 'site';
					$path = "$this->src/modules/$folder/$module";
					if(!is_dir($path)) {
						$path = "$this->src/modules/$folder/mod_$module";
					}
					if(!is_dir($path)) {
						$path = "$this->src/modules/$module";
					}
					if(!is_dir($path)) {
						$path = "$this->src/modules/mod_$module";
					}
					if(!is_dir($path)) continue;

					$installer = new JInstaller;
					$result = $installer->install($path);
					$this->status->modules[] = array('name'=>$module, 'client'=>$folder, 'result'=>$result);
				}
			}
		}
	}

	protected function installPlugins()
	{
		if(count($this->installation_queue['plugins'])) {
			foreach($this->installation_queue['plugins'] as $folder => $plugins) {
				if(count($plugins)) foreach($plugins as $plugin => $published) {
					$path = "$this->src/plugins/$folder/$plugin";
					if(!is_dir($path)) {
						$path = "$this->src/plugins/$folder/plg_$plugin";
					}
					if(!is_dir($path)) {
						$path = "$this->src/plugins/$plugin";
					}
					if(!is_dir($path)) {
						$path = "$this->src/plugins/plg_$plugin";
					}
					if(!is_dir($path)) continue;

					$installer = new JInstaller;
					$result = $installer->install($path);
					$this->status->plugins[] = array('name'=>'plg_'.$plugin,'group'=>$folder, 'result'=>$result);
				}
			}
		}
	}

	/**
	 * Renders the post-installation message
	 */
	private function _renderPostInstallation($status, $fofStatus, $parent)
	{
		?>

	<?php $rows = 0;?>
	<img src="../media/com_domus/images/domus-48.png" width="48" height="48" alt="Domus Organizer" align="right" />

	<h2>Welcome to Domus Organizer!</h2>

	<p>
		Congratulations! Now you can start using Domus Organizer!<br />
		We strongly suggest you <a href="index.php?option=com_domus&view=utilities&task=sampleData">to install sample data</a>,
		since it will install some required information and few examples.
	</p>
	<p style="font-weight:bold">
		WARNING! Sample data will delete and replace any existant data!
	</p>

	<table class="adminlist">
		<thead>
			<tr>
				<th class="title" colspan="2">Extension</th>
				<th width="30%">Status</th>
			</tr>
		</thead>
		<tfoot>
			<tr>
				<td colspan="3"></td>
			</tr>
		</tfoot>
		<tbody>
			<tr class="row0">
				<td class="key" colspan="2">Domus Organizer component</td>
				<td><strong style="color: green">Installed</strong></td>
			</tr>
			<tr class="row1">
				<td class="key" colspan="2">
					<strong>Framework on Framework (FOF) <?php echo $fofStatus['version']?></strong> [<?php echo $fofStatus['date'] ?>]
				</td>
				<td><strong>
					<span style="color: <?php echo $fofStatus['required'] ? ($fofStatus['installed']?'green':'red') : '#660' ?>; font-weight: bold;">
						<?php echo $fofStatus['required'] ? ($fofStatus['installed'] ?'Installed':'Not Installed') : 'Already up-to-date'; ?>
					</span>
				</strong></td>
			</tr>
			<?php if (count($this->status->modules)) : ?>
			<tr>
				<th>Module</th>
				<th>Client</th>
				<th></th>
			</tr>
			<?php foreach ($this->status->modules as $module) :
					$color = $module['result'] == 'Installed' ? 'green' : 'red';
			?>
			<tr class="row<?php echo (++ $rows % 2); ?>">
				<td class="key"><?php echo $module['name']; ?></td>
				<td class="key"><?php echo ucfirst($module['client']); ?></td>
				<td><strong style="color:<?php echo $color?>"><?php echo ($module['result'])?'Installed':'Not installed'; ?></strong></td>
			</tr>
			<?php endforeach;?>
			<?php endif;?>
			<?php if (count($this->status->plugins)) : ?>
			<tr>
				<th>Plugin</th>
				<th>Group</th>
				<th></th>
			</tr>
			<?php foreach ($this->status->plugins as $plugin) :
					$color = $plugin['result'] == 'Installed' ? 'green' : 'red';
			?>
			<tr class="row<?php echo (++ $rows % 2); ?>">
				<td class="key"><?php echo ucfirst($plugin['name']); ?></td>
				<td class="key"><?php echo ucfirst($plugin['group']); ?></td>
				<td><strong style="color:<?php echo $color?>"><?php echo ($plugin['result'])?'Installed':'Not installed'; ?></strong></td>
			</tr>
			<?php endforeach; ?>
			<?php endif; ?>
		</tbody>
	</table>
	<?php
		}

	/**
	 * Joomla! 1.6+ bugfix for "DB function returned no error"
	 */
	private function _bugfixDBFunctionReturnedNoError()
	{
		$db = JFactory::getDbo();

		// Fix broken #__assets records
		$query = $db->getQuery(true);
		$query->select('id')
			  ->from('#__assets')
			  ->where($db->qn('name').' = '.$db->q($this->_fabbrica_extension));
		$db->setQuery($query);
		$ids = $db->loadColumn();
		if(!empty($ids)) foreach($ids as $id) {
			$query = $db->getQuery(true);
			$query->delete('#__assets')
				  ->where($db->qn('id').' = '.$db->q($id));
			$db->setQuery($query);
			$db->query();
		}

		// Fix broken #__extensions records
		$query = $db->getQuery(true);
		$query->select('extension_id')
			  ->from('#__extensions')
			  ->where($db->qn('element').' = '.$db->q($this->_fabbrica_extension));
		$db->setQuery($query);
		$ids = $db->loadColumn();
		if(!empty($ids)) foreach($ids as $id) {
			$query = $db->getQuery(true);
			$query->delete('#__extensions')
				  ->where($db->qn('extension_id').' = '.$db->q($id));
			$db->setQuery($query);
			$db->query();
		}

		// Fix broken #__menu records
		$query = $db->getQuery(true);
		$query->select('id')
			  ->from('#__menu')
			  ->where($db->qn('type').' = '.$db->q('component'))
			  ->where($db->qn('menutype').' = '.$db->q('main'))
			  ->where($db->qn('link').' LIKE '.$db->q('index.php?option='.$this->_fabbrica_extension));
		$db->setQuery($query);
		$ids = $db->loadColumn();
		if(!empty($ids)) foreach($ids as $id) {
			$query = $db->getQuery(true);
			$query->delete('#__menu')
				  ->where($db->qn('id').' = '.$db->q($id));
			$db->setQuery($query);
			$db->query();
		}
	}

	/**
	 * Joomla! 1.6+ bugfix for "Can not build admin menus"
	 */
	private function _bugfixCantBuildAdminMenus()
	{
		$db = JFactory::getDbo();

		// If there are multiple #__extensions record, keep one of them
		$query = $db->getQuery(true);
		$query->select('extension_id')
			  ->from('#__extensions')
			  ->where($db->qn('element').' = '.$db->q($this->_fabbrica_extension));
		$db->setQuery($query);
		$ids = $db->loadColumn();
		if(count($ids) > 1) {
			asort($ids);
			$extension_id = array_shift($ids); // Keep the oldest id

			foreach($ids as $id) {
				$query = $db->getQuery(true);
				$query->delete('#__extensions')
					  ->where($db->qn('extension_id').' = '.$db->q($id));
				$db->setQuery($query);
				$db->query();
			}
		}

		// @todo

		// If there are multiple assets records, delete all except the oldest one
		$query = $db->getQuery(true);
		$query->select('id')
			  ->from('#__assets')
			  ->where($db->qn('name').' = '.$db->q($this->_fabbrica_extension));
		$db->setQuery($query);
		$ids = $db->loadObjectList();
		if(count($ids) > 1) {
			asort($ids);
			$asset_id = array_shift($ids); // Keep the oldest id

			foreach($ids as $id) {
				$query = $db->getQuery(true);
				$query->delete('#__assets')
					  ->where($db->qn('id').' = '.$db->q($id));
				$db->setQuery($query);
				$db->query();
			}
		}

		// Remove #__menu records for good measure!
		$query = $db->getQuery(true);
		$query->select('id')
			  ->from('#__menu')
			  ->where($db->qn('type').' = '.$db->q('component'))
			  ->where($db->qn('menutype').' = '.$db->q('main'))
			  ->where($db->qn('link').' LIKE '.$db->q('index.php?option='.$this->_fabbrica_extension));
		$db->setQuery($query);
		$ids1 = $db->loadColumn();
		if(empty($ids1)) $ids1 = array();
		$query = $db->getQuery(true);
		$query->select('id')
			  ->from('#__menu')
			  ->where($db->qn('type').' = '.$db->q('component'))
			  ->where($db->qn('menutype').' = '.$db->q('main'))
			  ->where($db->qn('link').' LIKE '.$db->q('index.php?option='.$this->_fabbrica_extension.'&%'));
		$db->setQuery($query);
		$ids2 = $db->loadColumn();
		if(empty($ids2)) $ids2 = array();
		$ids = array_merge($ids1, $ids2);
		if(!empty($ids)) foreach($ids as $id) {
			$query = $db->getQuery(true);
			$query->delete('#__menu')
				  ->where($db->qn('id').' = '.$db->q($id));
			$db->setQuery($query);
			$db->query();
		}
	}

	private function _installFOF($parent)
	{
		$src = $parent->getParent()->getPath('source');

		// Install the FOF framework
		jimport('joomla.filesystem.folder');
		jimport('joomla.filesystem.file');
		jimport('joomla.utilities.date');
		$source = $src.'/fof';
		if(!defined('JPATH_LIBRARIES')) {
			$target = JPATH_ROOT.'/libraries/fof';
		} else {
			$target = JPATH_LIBRARIES.'/fof';
		}
		$haveToInstallFOF = false;
		if(!JFolder::exists($target)) {
			$haveToInstallFOF = true;
		} else {
			$fofVersion = array();
			if(JFile::exists($target.'/version.txt')) {
				$rawData = JFile::read($target.'/version.txt');
				$info = explode("\n", $rawData);
				$fofVersion['installed'] = array(
						'version'	=> trim($info[0]),
						'date'		=> new JDate(trim($info[1]))
				);
			} else {
				$fofVersion['installed'] = array(
						'version'	=> '0.0',
						'date'		=> new JDate('2011-01-01')
				);
			}
			$rawData = JFile::read($source.'/version.txt');
			$info = explode("\n", $rawData);
			$fofVersion['package'] = array(
					'version'	=> trim($info[0]),
					'date'		=> new JDate(trim($info[1]))
			);

			$haveToInstallFOF = $fofVersion['package']['date']->toUNIX() > $fofVersion['installed']['date']->toUNIX();
		}

		$installedFOF = false;
		if($haveToInstallFOF) {
			$versionSource = 'package';
			$installer = new JInstaller;
			$installedFOF = $installer->install($source);
		} else {
			$versionSource = 'installed';
		}

		if(!isset($fofVersion)) {
			$fofVersion = array();
			if(JFile::exists($target.'/version.txt')) {
				$rawData = JFile::read($target.'/version.txt');
				$info = explode("\n", $rawData);
				$fofVersion['installed'] = array(
						'version'	=> trim($info[0]),
						'date'		=> new JDate(trim($info[1]))
				);
			} else {
				$fofVersion['installed'] = array(
						'version'	=> '0.0',
						'date'		=> new JDate('2011-01-01')
				);
			}
			$rawData = JFile::read($source.'/version.txt');
			$info = explode("\n", $rawData);
			$fofVersion['package'] = array(
					'version'	=> trim($info[0]),
					'date'		=> new JDate(trim($info[1]))
			);
			$versionSource = 'installed';
		}

		if(!($fofVersion[$versionSource]['date'] instanceof JDate)) {
			$fofVersion[$versionSource]['date'] = new JDate();
		}

		return array(
				'required'	=> $haveToInstallFOF,
				'installed'	=> $installedFOF,
				'version'	=> $fofVersion[$versionSource]['version'],
				'date'		=> $fofVersion[$versionSource]['date']->toFormat('%Y-%m-%d'),
		);
	}
}